use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath LoadRootObject LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Order::API::Order qw ( CreateOrder );
use DE_EPAGES::Shop::API::Table::ShopCurrency qw (UpdateShopCurrency);
use DE_EPAGES::AvaTax::API::AvaTax qw ( UseAvaTax );
use DE_EPAGES::AvaTax::API::Constants qw (
    AVATAX_DOCUMENTTYPE_SalesOrder
);
use strict;
use feature 'switch';

my $ROOTPATH = '/Test';

sub run_tests {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            # initializsation
            ok( Init(), 'initialization' );
            eval {
                testImportXML();
                testGenerateOrder();
            };
            if (ExistsError()) {
                my $Error = GetError();
                fail('Error - '. $Error->code);
                diag($Error->longMessage);
                # always cleanup
                ok( Cleanup(), 'cleanup');
                Error($Error);
            }
            # cleanup
            ok( Cleanup(), 'cleanup');
        }
    );
}

sub testImportXML {
    ### Validate.
    my $Shop = LoadObjectByPath( $ROOTPATH . '/TestShopAvaTax' );
    # disable additional currencies.
    map { UpdateShopCurrency($Shop->id,$_->{'CurrencyID'},{'IsActivated' => 0}) }
        grep { $_->{'IsActivated'} && $_->{'CurrencyID'} ne 'USD'}
        @{$Shop->get('Currencies')};
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    my $TaxMethod = $Shop->get('TaxMethod');
    isa_ok($TaxMethod, 'DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax');
    my $LineItemTaxClass = $Shop->get('TaxMatrix')->get('LineItemClass');
    isa_ok($LineItemTaxClass, 'DE_EPAGES::Object::API::Object::Class');
    is($LineItemTaxClass->alias , 'LineItemAvaTax', 'Class isa LineItemAvaTax');
    is($TaxMethod->usableAtStorefront(), 1, 'TaxMethod is usable at storefront.');
    my $Customer = $Shop->path('Customers/TestCustomer');
    isa_ok($Customer, 'DE_EPAGES::Customer::API::Object::Customer');
    my $Address = $Customer->get('BillingAddress');
    isa_ok($Address, 'DE_EPAGES::Address::API::Object::Address');
    my $Request = DE_EPAGES::AvaTax::API::AvaTax::_CreateValidateRequestFromAddress($Address); ## no critic (ProtectPrivateSubs)
    isa_ok($Request, 'DE_EPAGES::AvaTax::API::DTO::ValidateRequest');
    my $DTOAddress = $Request->popAddress();
    isa_ok($DTOAddress, 'DE_EPAGES::AvaTax::API::DTO::Address');
    is($DTOAddress->Line1(), 'ePages GmbH', 'Validate Line1');
    is($DTOAddress->Line2(), '3130 Elliott', 'Validate Line2');
    is($DTOAddress->City(), 'Seattle', 'Validate City');
    is($DTOAddress->PostalCode(), '98121', 'Validate PostalCode');
    is($DTOAddress->Country(), 'USA', 'Validate Country');

    ### GetTax.
    my $Order = $Shop->path('Customers/TestCustomer/Orders/1234-5678');
    isa_ok($Order, 'DE_EPAGES::Order::API::Object::CustomerOrder');
    my $LineItemContainer = $Order->child('LineItemContainer');
    isa_ok($LineItemContainer, 'DE_EPAGES::Order::API::Object::LineItemContainer');
    $Request = DE_EPAGES::AvaTax::API::AvaTax::_CreateGetTaxRequest($LineItemContainer, {'AvaTax_CompanyCode' => 'EPAGE'}); ## no critic (ProtectPrivateSubs)
    isa_ok($Request, 'DE_EPAGES::AvaTax::API::DTO::GetTaxRequest');
    is($Request->CompanyCode(), 'EPAGE', 'GetTax CompanyCode');
    is($Request->DocType(), AVATAX_DOCUMENTTYPE_SalesOrder, 'GetTax DocType');
    is($Request->OriginCode(), 'Origin', 'GetTax OriginCode');
    is($Request->DestinationCode(), 'Dest', 'GetTax DestinationCode');
    is(scalar @{$Request->AddressList()}, 2, 'GetTax AddressList');
    is(scalar @{$Request->LineList()}, 5, 'GetTax LineList');

    my ($Source, $Dest) = @{$Request->AddressList()};
    is($Source->Line1(), 'Avalara', 'GetTax Source Line1');
    is($Source->Line2(), '900 Winslow Way', 'GetTax Source Line2');
    is($Source->Line3(), 'Suite 100', 'GetTax Source Line3');
    is($Source->City(), 'Bainbridge Island', 'GetTax Source City');
    is($Source->PostalCode(), '98110', 'GetTax Source PostalCode');
    is($Source->Country(), 'USA', 'GetTax Source Country');

    is($Dest->Line1(), 'ePages GmbH', 'GetTax Dest Line1');
    is($Dest->Line2(), '3130 Elliott', 'GetTax Dest Line2');
    is($Dest->City(), 'Seattle', 'GetTax Dest City');
    is($Dest->PostalCode(), '98121', 'GetTax Dest PostalCode');
    is($Dest->Country(), 'USA', 'GetTax Dest Country');
}

sub testGenerateOrder {
    my $Shop = LoadObjectByPath( $ROOTPATH . '/TestShopAvaTax' );
    isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop');
    my $Customer = $Shop->path('Customers/TestCustomer');

    my $Order = CreateOrder($Customer,{
        'Alias' => '246810',
    });

    ok( $Order->id, 'insert new order' );

    my $Container = $Order->get('LineItemContainer');

    my $Product = $Shop->path('Products/MyPetrol');
    isa_ok($Product, 'DE_EPAGES::Product::API::Object::Product');
    my %ProductInfo = (
        'Quantity'          => 2,
        'CurrencyID'        => 'USD',
        'Name'              => 'MyPetrol',
    );
    $Product->addToContainer($Container, \%ProductInfo);
    $Product->addToContainer($Container, \%ProductInfo);
    # 4 x $0.25 = $1.00 net.

    $Product = $Shop->path('Products/MyProduct');
    isa_ok($Product, 'DE_EPAGES::Product::API::Object::Product');
    %ProductInfo = (
        'Quantity'          => 1,
        'CurrencyID'        => 'USD',
        'Name'              => 'MyProduct',
    );
    $Product->addToContainer($Container, \%ProductInfo);
    # 1x $0.50 = $0.50 net

    $Product = $Shop->path('Products/MyGas');
    isa_ok($Product, 'DE_EPAGES::Product::API::Object::Product');
    %ProductInfo = (
        'Quantity'          => 10,
        'Name'              => 'MyGas',
    );
    $Product->addToContainer($Container, \%ProductInfo);
    # 10x $1.00 = $10.00 net

    # additional amounts:
    # $11.76 shipping
    # total = $23.26 net

    $Order->insertChild('Address',{
        'Alias' => 'ShippingAddress',
        'DisplayName' => 'Thomas TM Bergmann',
        'Company' => 'ePages GmbH',
        'Street' => '3130 Elliott',
        'Zipcode' => '98121',
        'City' => 'Seattle',
        'CountryID' => '840',
    });

    # double check if AvaTax will be triggered.
    ok(UseAvaTax($Shop), 'UseAvaTax');

    # this triggers a hook in
    $Order->recalculate;
    # so we have
    my @LineItemTaxes = @{$Container->get('Taxes')};
    is(scalar @LineItemTaxes, 3, '3x lineitemtax');
    foreach my $LineItemTax (@LineItemTaxes){
        isa_ok($LineItemTax, 'DE_EPAGES::AvaTax::API::Object::LineItemAvaTax');
        given ($LineItemTax->get('TaxClass')->alias) {
            when ('P0000000') {
                cmp_ok($LineItemTax->get('TaxAmount'), '==', 0.15,  'TaxAmount P0000000');
                cmp_ok($LineItemTax->get('TaxRate'),   '==', 0.095, 'TaxRate P0000000');
            }
            when ('PC040400') {
                cmp_ok($LineItemTax->get('TaxAmount'), '==', 0.95,  'TaxAmount PC040400');
                cmp_ok($LineItemTax->get('TaxRate'),   '==', 0.095, 'TaxRate PC040400')
            }
            when ('FR') {
                cmp_ok($LineItemTax->get('TaxAmount'), '==', 1.11,  'TaxAmount FR');
                cmp_ok($LineItemTax->get('TaxRate'),   '==', 0.095, 'TaxRate FR')
            }
            default {
                fail('unknown taxclass '.$_);
            }
       }
    }
    cmp_ok($Container->get('TotalBeforeTax'),'==',23.26, 'TotalBeforeTax');
    cmp_ok($Container->get('TotalTax'),      '==', 2.21, 'TotalTax');
    cmp_ok($Container->get('GrandTotal'),    '==',25.47, 'GrandTotal');
}

sub Main {
    run_tests;
    done_testing;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    # set back to old value.
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

1;
