﻿use strict;
use utf8;
use Test::More tests => 87;
use DE_EPAGES::Content::API::ShortUrl qw ( GetObjectByURI GenerateShortUrl );
use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::IDN qw (EncodeDomainInUrl EncodeDomain);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Constants qw ( EXPANDWWWDOMAIN );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', 'Shop loaded' );

    my $RootCategory = $Shop->get('Categories');
    $RootCategory->child('TEST')->delete if $RootCategory->existsChild('TEST');

    #Category without shortUrl
    my $TestCategory = $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST' });
    ok ($TestCategory->get('URI') eq '', 'category without shortUrl');
    ok (defined $TestCategory->get('CanonicalURL'), 'defined CanonicalURL from category without shortUrl');

    #set shortUrl
    $TestCategory->set({'URI' => 'Test'});
    ok ($TestCategory->get('URI') eq 'Test', 'category with shortUrl');
    ok (defined $TestCategory->get('CanonicalURL'), 'defined CanonicalURL from category with shortUrl');

    #set domain name
    my $LocaleID = $Shop->get('LocaleID');
    is ($TestCategory->get('CanonicalURL'), $Shop->get('ProtocolAndServer').'/epages/Store.sf/'.$LocaleID.'/?ObjectPath=/Test/TestShop/Categories/TEST', 'CanonicalURL with objectpath');
    $Shop->set({'DomainName' => 'www.myshop.de'});
    ok ($TestCategory->get('CanonicalURL') eq 'http://www.myshop.de/Test', 'CanonicalURL with shortUrl');

    #find category by url
    my ($ContentObject) = GetObjectByURI($Shop, 'Test');
    ok ($ContentObject->id == $TestCategory->id, 'find category by shortUrl');
    ok (!defined GetObjectByURI($Shop, 'NotExists'), 'cannot find category by unknown shortUrl');

    #find category by upper and lower case
    ($ContentObject) = GetObjectByURI($Shop, 'test');
    ok ($ContentObject->id == $TestCategory->id, 'find category by shortUrl in lower case');
    ($ContentObject) = GetObjectByURI($Shop, 'TEST');
    ok ($ContentObject->id == $TestCategory->id, 'find category by shortUrl in upper case');

    #update shortUrl
    $TestCategory->set({'URI' => 'Test/New'});
    ok ($TestCategory->get('URI') eq 'Test/New', 'update shortUrl');
    ok (!defined GetObjectByURI($Shop, 'test'), 'cannot find category by old shortUrl after update');
    ok (!defined GetObjectByURI($Shop, 'Test'), 'cannot find category by old shortUrl after update');
    ($ContentObject) = GetObjectByURI($Shop, 'test/New');
    ok ($ContentObject->id == $TestCategory->id, 'find category by new shortUrl after update');

    #delete shortUrl (set undef)
    $TestCategory->set({'URI' => undef});
    ok (! defined $TestCategory->get('URI'), 'delete shortUrl by setting undef');
    $TestCategory->set({'URI' => ''});
    ok (! defined $TestCategory->get('URI'), 'delete shortUrl by setting empty');

    #check length
    eval { $TestCategory->set({'URI' => '1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456'}); };
    ok (ExistsError && GetError()->code eq 'STRING_TOO_LONG', 'cannot set url with more than 255 characters');

    eval { $TestCategory->set({'URI' => '123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345'}); };
    ok (!ExistsError, 'can set url with 255 characters');

    #create new category with same shortUrl
    $TestCategory->set({'URI' => 'Test'});
    eval { $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST2', URI => 'TEST' }); };
    ok (ExistsError && GetError()->code eq 'SHORTURL_ALREADY_EXISTS', 'url already exists');
    eval { $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST2', URI => 'Test' }); };
    ok (ExistsError && GetError()->code eq 'SHORTURL_ALREADY_EXISTS', 'url already exists');

    #check valid characters
    eval { $TestCategory->set({'URI' => 'Test-6/Test'}) };
    ok (!ExistsError, 'valid characters');
    eval { $TestCategory->set({'URI' => '/Test-6/Test'}) };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters: cannot start with "/"');
    eval { $TestCategory->set({'URI' => 'Test_1'}); };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters');
    eval { $TestCategory->set({'URI' => 'Test 1'}); };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters');
    eval { $TestCategory->set({'URI' => 'Test%201'}); };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters');

    #cannot set shortUrl for startpage
    eval { $RootCategory->set({'URI' => 'Start'}); };
    ok (ExistsError && GetError()->code eq 'ShortURL_NOT_ALLOWED', 'cannot set shortUrl for startpage');

    #generate short url
    $TestCategory->set({'URI' => 'Test'});
    ok (GenerateShortUrl($Shop, 'Test123') eq 'Test123', 'generate shortUrl Test123');
    ok (GenerateShortUrl($Shop, 'Test 123_456') eq 'Test-123456', 'generate shortUrl Test-123456');
    ok (GenerateShortUrl($Shop, 'Hündchenölgemälde mit Straße') eq 'Huendchenoelgemaelde-mit-Strasse', 'generate shortUrl Huendchenoelgemaelde-mit-Strasse');
    ok (GenerateShortUrl($Shop, ' test @ test') eq 'test-test', 'generate shortUrl test-test');
    ok (GenerateShortUrl($Shop, 'Ποσειδων') eq 'Poseidon', 'generate shortUrl Poseidon');
    ok (GenerateShortUrl($Shop, '  ∫√ﬁ∆ℓ') eq '', 'generate shortUrl (empty)');
    ok (GenerateShortUrl($Shop, 'Test') eq 'Test-1', 'generate shortUrl with existing url');
    ok (GenerateShortUrl($Shop, '/Test/1/') eq 'Test/1', 'generate shortUrl with /');

    #with temporary list
    my $ahTemporary = [{'Uri' => 'Test1'}, {'Uri' => 'Test/en'}];
    ok (GenerateShortUrl($Shop, 'Test1', 1, $ahTemporary) eq 'Test1/de', 'generate shortUrl with existing url in temporary list');
    ok (GenerateShortUrl($Shop, 'Test', 2, $ahTemporary) eq 'Test-1/en', 'generate shortUrl with existing url in temporary list');

    #generate short url with 250 characters (+ counter)
    ok (
        GenerateShortUrl(
            $Shop,
            'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.'
        ) eq 'Lorem-ipsum-dolor-sit-amet-consetetur-sadipscing-elitr-sed-diam-nonumy-eirmod-tempor-invidunt-ut-labore-et-dolore-magna-aliquyam-erat-sed-diam-voluptua-At-vero-eos-et-accusam-et-justo-duo-dolores-et-ea-rebum-Stet-clita-kasd-gubergren-no-sea-takimata', 'generate shortUrl from long name'
    );
    $TestCategory->set({'URI' => 'Lorem-ipsum-dolor-sit-amet-consetetur-sadipscing-elitr-sed-diam-nonumy-eirmod-tempor-invidunt-ut-labore-et-dolore-magna-aliquyam-erat-sed-diam-voluptua-At-vero-eos-et-accusam-et-justo-duo-dolores-et-ea-rebum-Stet-clita-kasd-gubergren-no-sea-takimata'});
    ok (
        GenerateShortUrl(
            $Shop,
            'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.'
        ) eq 'Lorem-ipsum-dolor-sit-amet-consetetur-sadipscing-elitr-sed-diam-nonumy-eirmod-tempor-invidunt-ut-labore-et-dolore-magna-aliquyam-erat-sed-diam-voluptua-At-vero-eos-et-accusam-et-justo-duo-dolores-et-ea-rebum-Stet-clita-kasd-gubergren-no-sea-takimata-1', 'generate shortUrl from long name'
    );
    ok (
        GenerateShortUrl(
            $Shop,
            'Lorem/ipsum/dolor sit amet/consetetur/sadipscing elitr'
        ) eq 'Lorem/ipsum/dolor-sit-amet/consetetur/sadipscing-elitr', 'generate shortUrl with path'
    );

    # test for EPG-15738: redirect www-Domain to primary domain
    my $EpagesConf = EpagesConfig();
    $EpagesConf->set('DE_EPAGES::Object', 'ExpandWWWDomain', 1);
    ok ( EXPANDWWWDOMAIN, 'ExpandWWWDomain' );

    CheckRedirects($Shop, 'testshop.lan');
    CheckRedirects($Shop, EncodeDomain('grünerübengärtchen.lan'));
    CheckRedirects($Shop, EncodeDomain('café.lan'));

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    $Shop->set({'DomainName' => 'myothertestshop.lan'});
    cmp_ok( $FakeRequest->get('http://myothertestshop.lan')->code, '==', 200, 'Access to non-www domain does no redirect when shop has non-www domain assigned' );
    my $Response = $FakeRequest->get('http://www.myothertestshop.lan');
    ok( $Response->code == 301 && $Response->header('Location') eq 'http://myothertestshop.lan/', 'Access to www domain does 301 redirection to non-www domain' );

    my $Gen = GenerateShortUrl($Shop, 'MyOtherTestCategory123');
    $TestCategory->set({'URI' => $Gen});
    cmp_ok( $TestCategory->get('URI'), 'eq', $Gen, 'Set test category shorturl');
    cmp_ok( $TestCategory->get('CanonicalURL'), 'eq', 'http://myothertestshop.lan/MyOtherTestCategory123', 'Canonical URL ok');
    $Response = $FakeRequest->get('http://www.myothertestshop.lan/MyOtherTestCategory123');
    ok( $Response->code == 301 && $Response->header('Location') eq EncodeDomainInUrl('http://myothertestshop.lan/MyOtherTestCategory123'), 'Access to www URL does 301 redirection to non-www URL' );

    my $ShopSSLHostName = $Shop->get('WebUrlSSL');
    SKIP: {
        skip 'SSL not enabled', 3 unless $ShopSSLHostName =~ /^https/;
        my $SessionURL = LoginToshop($Shop, $Shop->get('WebUrlSSL'));
        $Response = $FakeRequest->get($SessionURL);
        cmp_ok( $FakeRequest->get('http://myothertestshop.lan')->code, '==', 200, 'Access to https domain with session does no redirect' );
    }

    # add languages & locales
    my $LanguageID = GetPKeyLanguageByCode( 'en' );
    $Shop->addLanguage($LanguageID);
    ok ( $Shop->existsLanguage($LanguageID) , 'existsLanguage 2' );
    ok ( 1 == scalar @{$Shop->languages}, 'Shop.languages 2' );
    ok ( $LanguageID == $Shop->languages->[0], 'Shop.languages[0] 2' );
    ok ( 'en_GB' eq $Shop->get('LocaleID'), 'Shop.LocaleID 2' );

    $Shop->featureMaxValue('Languages', 2);
    $LanguageID = GetPKeyLanguageByCode( 'de' );
    $Shop->addLanguage($LanguageID);
    ok( $Shop->existsLanguage($LanguageID) , 'existsLanguage(de)' );
    ok( 2 == scalar @{$Shop->languages}, 'Shop.languages = 2' );
    ok( 'en_GB' eq $Shop->get('LocaleID'), 'Shop.LocaleID does not change' );

    $Shop->addLocale('en_AU');
    ok( $Shop->existsLocale('en_AU'), 'addLocale en_AU' );

    $Shop->addLocale('de_AT');
    ok( $Shop->existsLocale('de_AT'), 'addLocale de_AT' );

    $Shop->updateSiteLocale('en_AU', 1);
    $Shop->updateSiteLocale('de_AT', 2);
    $Shop->set({'LocaleID' => 'de_AT'});

    $Shop->removeLocale('de_DE');
    $Shop->removeLocale('en_GB');

    my $ShopLocales = $Shop->get('SiteLocales');
    cmp_ok ( scalar(@$ShopLocales), '==', 2, 'Shop has now 2 locales');

    # pages without generated shorturl should use correct locale id
    my $LocTestCat = $Shop->get('Categories')->insertChild('Category', { Alias => 'LocTestCat' });
    cmp_ok($LocTestCat->get('URI'), 'eq', '', 'category without shortUrl');
    cmp_ok( $LocTestCat->get('CanonicalURL', 1),
            'eq',
            $Shop->get('WebServerScriptUrl') . '/de_AT/?ObjectPath=/Test/TestShop/Categories/LocTestCat',
            'CanonicalURL from category without shortUrl'
    );
    cmp_ok( $LocTestCat->get('CanonicalURL', 2),
            'eq',
            $Shop->get('WebServerScriptUrl') . '/en_AU/?ObjectPath=/Test/TestShop/Categories/LocTestCat',
            'CanonicalURL from category without shortUrl'
    );

    # pages with shorturl should point to the correct language of the page
    $LocTestCat->set({'URI' => 'loc-test-cat-ger'}, 1);
    $LocTestCat->set({'URI' => 'loc-test-cat-eng'}, 2);
    cmp_ok( $LocTestCat->get('CanonicalURL', 1),
            'eq',
            'http://'.$Shop->get('DomainName') . '/loc-test-cat-ger',
            'CanonicalURL from category with shortUrl'
    );
    cmp_ok( $LocTestCat->get('CanonicalURL', 2),
            'eq',
            'http://'.$Shop->get('DomainName') . '/loc-test-cat-eng',
            'CanonicalURL from category with shortUrl'
    );
    my $LocResponse = $FakeRequest->get($LocTestCat->get('CanonicalURL', 1));
    ok( $LocResponse->code == 200, 'Access to to shorturl ok' );

    my ($BaseTag) = $LocResponse->content =~ /baseUrl\s*=\s*['"](.*?)['"]/sx;     #'
    cmp_ok($BaseTag, 'eq', $Shop->get('WebServerScriptUrl').'/de_AT/', 'Content page in correct language');
    my ($CanonicalTagUrl) = $LocResponse->content =~ /<link\srel="canonical"\shref="(.*?)"\s\/>/mgx;
    cmp_ok($CanonicalTagUrl, 'eq', $LocTestCat->get('CanonicalURL', 1), 'Canonical tag URL ok');

    $LocResponse = $FakeRequest->get($LocTestCat->get('CanonicalURL', 2));
    ok( $LocResponse->code == 200, 'Access to to shorturl ok' );
    ($BaseTag) = $LocResponse->content =~ /baseUrl\s*=\s*['"](.*?)['"]/sx;     #'
    cmp_ok($BaseTag, 'eq', $Shop->get('WebServerScriptUrl').'/en_AU/', 'Content page in correct language');
    ($CanonicalTagUrl) = $LocResponse->content =~ /<link\srel="canonical"\shref="(.*?)"\s\/>/mgx;
    cmp_ok($CanonicalTagUrl, 'eq', $LocTestCat->get('CanonicalURL', 2), 'Canonical tag URL ok');

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub CheckRedirects {
    my ($Shop, $TestURL) = @_;

    $Shop->set({'DomainName' => 'www.'.$TestURL});
    my $ShopSSLHostName = $Shop->get('WebUrlSSL');
    my $StoreCanSSL = $ShopSSLHostName =~ /^https/ ? 1 : 0;

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    cmp_ok( $FakeRequest->get('http://www.'.$TestURL)->code, '==', 200, 'Access to www domain does no redirect' );

    my $Response = $FakeRequest->get('http://'.$TestURL);
    ok( $Response->code == 301 && $Response->header('Location') eq EncodeDomainInUrl('http://www.'.$TestURL.'/'), 'Access to non-www domain does 301 redirection to www domain' );

    SKIP: {
        skip 'SSL not enabled', 3 unless $StoreCanSSL;

        $Response = $FakeRequest->get('https://www.'.$TestURL);
        ok( $Response->code == 301 && $Response->header('Location') eq EncodeDomainInUrl('http://www.'.$TestURL.'/'), 'Access to www https domain does 301 redirection to http domain' );

        $Response = $FakeRequest->get('https://'.$TestURL);
        ok( $Response->code == 301 && $Response->header('Location') eq EncodeDomainInUrl('http://www.'.$TestURL.'/'), 'Access to non-www https domain does 301 redirection to http www domain' );

        $Response = $FakeRequest->get($ShopSSLHostName);
        my $PathQuery = URI->new($Response->header('Location'))->path_query;
        ok( $Response->code == 301 && $Response->header('Location') eq EncodeDomainInUrl('http://www.'.$TestURL.$PathQuery), 'Access to shop\'s ssl url without session does 301 redirection to shop http www domain' );
    }
}

sub LoginToshop {
    my ($Shop, $URL) = @_;

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $UrlParams = join('&', (
        'ViewAction=View',
        'ChangeAction=SaveLoginForm',
        'Login=mmustermann',
        'Password=123456'
    ));

    my $Response = $FakeRequest->get(
        $URL . '&' . $UrlParams,
    );

    cmp_ok( $Response->code, '==', 200, 'SF login to shop response 200 OK' );

    # grep URL with sec-Token inside from content (LWP::UserAgent would do this aswell ...)
    # i.e.: <base href="https://qa-vm-lin64-1.intern.epages.de/epages/Store.admin/de_DE/">
    my ($SessionURL) = $Response->content =~ /<base\s*href=["'](.*)["']>/mgx;   #'
    if(!$SessionURL || $SessionURL eq '') {
        # alternatively we have a javascript var for that URL
        # i.e.: epages.vars.BaseUrl = 'http://slange.intern.epages.de/epages/DemoShop.admin/sec7f9ebdacf6/';
        ($SessionURL) = $Response->content =~ /['"]baseUrl['"]\s*:\s*['"](.*)['"]/mgx;     #'
    }
    ok( $SessionURL =~ /Store\.sf\/sec/g, 'Search for session URL in content');

    return $SessionURL;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);