#!perl -w
use Test::More;
use strict;
use Config;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::File qw ( WriteFile CreateDirectory DeleteDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Config qw(StoreConnections BUStoreTypes StoreTypesWithCartridge ShopStoreTypes SiteStoreTypes);

my $CartridgesXml = <<'END_FILE',
<?xml version="1.0" encoding="UTF-8"?>
<epages>
  <DatabaseType Name="ASPoolDB" >
    <Cartridge Package="DE_EPAGES::Core" />
    <Cartridge Package="DE_EPAGES::XML" />
  </DatabaseType>
  <DatabaseType Name="Site">
    <Cartridge Package="DE_EPAGES::Store" />
    <Cartridge Package="DE_EPAGES::ShopConfiguration" />
  </DatabaseType>
  <DatabaseType Name="Store">
    <Cartridge Package="DE_EPAGES::Core" />
    <Cartridge Package="DE_EPAGES::Shop" />
  </DatabaseType>
  <DatabaseType Name="Store2">
    <Cartridge Package="DE_EPAGES::Calendar" />
    <Cartridge Package="DE_EPAGES::Shop" />
  </DatabaseType>
</epages>
END_FILE

my %StoreConf = (
    ASPoolDB => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/ASPool.db
StoreType=ASPoolDB
END_FILE
    Backup => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
END_FILE
    EbayCategories => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/EbayCategories.db
END_FILE
    Site => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Site
Database=sitedb
END_FILE
    MyStore => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Store
Database=storedb
END_FILE
    MyStore2 => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Store2
END_FILE
    );

sub Main {

    plan tests => 5;

    my $ConfigDir = tempdir();
    my $DBDir = "$ConfigDir/Database.d";

    CreateDirectory($DBDir, { recursive => 1 });

    local $ENV{EPAGES_CONFIG} = $ConfigDir;
    local $ENV{EPAGES_CARTRIDGES} = $ConfigDir;

    WriteFile( "$DBDir/$_.conf", \$StoreConf{$_} ) foreach keys %StoreConf;
    WriteFile( "$ConfigDir/Cartridges.xml", \$CartridgesXml );

    my (@Expected, @Found);

    # test 1: find all store connections
    @Expected = sort keys %StoreConf;
    @Found = sort @{StoreConnections()};
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    # test 2: find all stores types with business unit
    @Expected = sort qw(Site Store);
    @Found = sort @{BUStoreTypes()};
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    # test 3: find all stores types with DE_EPAGES::Shop cartridge
    @Expected = sort qw(Store Store2);
    @Found = sort @{ShopStoreTypes()};
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    # test 4: find all site stores types
    @Expected = sort qw(Site);
    @Found = sort @{SiteStoreTypes()};
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    # test 5: find all stores types with cartridge "DE_EPAGES::Core"
    @Expected = sort qw(ASPoolDB Store);
    @Found = sort @{StoreTypesWithCartridge('DE_EPAGES::Core')};
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    DeleteDirectory($ConfigDir, { recursive => 1 });
}

RunScript(Sub=>\&Main);
