use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MeinPaket::API::Table::MeinPaketCustomerOrderMap qw (
  GetPKeyMeinPaketCustomerOrderMapByOrderID
  InsertMeinPaketCustomerOrderMap
);
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  JoinObjectPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use Readonly;
use Test::More;

Readonly my $ROOTALIAS => 'Test';
Readonly my $ROOTPATH  => '/' . $ROOTALIAS;

sub testOrder2Order {
    my $Order = LoadObjectByPath(
        JoinObjectPath( [ '', $ROOTALIAS, 'TestShopMeinPaket', 'Customers', 'TestCustomer', 'Orders', '1234-5678' ] ) );
    InsertMeinPaketCustomerOrderMap( { 'OrderID' => $Order->id, 'MeinPaketOrderID' => 'XY-Test' } );
    my $MeinPaketMap = GetPKeyMeinPaketCustomerOrderMapByOrderID( $Order->id );
    ok( defined $MeinPaketMap, 'meinpaket order info map exists' );

    my $EditableOrder = $Order->clone( {
            'Parent'        => $Order->get('Customer')->folder('EditableOrders'),
            'Inherit'       => 1,
            'OriginalOrder' => $Order
        },
        1
    );
    isa_ok( $EditableOrder, 'DE_EPAGES::Order::API::Object::CustomerOrder', 'EditableOrder' );
    my $Orders = $Order->parent;
    TriggerHook( 'BeforeEditOrder2Order', { 'Order' => $EditableOrder } );

    # delete the original order and set the parent from the copy to Orders
    $Order->delete;
    $EditableOrder->set( { 'Parent' => $Orders, 'OriginalOrder' => undef } );
    $MeinPaketMap = GetPKeyMeinPaketCustomerOrderMapByOrderID( $EditableOrder->id );
    ok( defined $MeinPaketMap, 'meinpaket order info map exists after order2order' );
}

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # initialization
            ok( Init(), 'initialization' );

            # tests
            eval { testOrder2Order(); };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'ERROR: ' . $Error->longMessage );
            }

            # cleanup
            ok( Cleanup(), 'cleanup' );
            done_testing;
            return;
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;
