use strict;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  JoinObjectPath
  LoadObjectByPath
  LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::MeinPaket::API::Object::RequestAdapter;
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

                # tests
                testSerializeRefUnitUndef();
                testSerializeRefUnitMililitre();
                testSerializeRefUnitPiece();

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );

    my $Config = _loadPortalConfig();

    # initialize with our test account.
    $Config->set( {
            'PortalMerchantID'       => 'cartridge@epages.com',
            'PortalMerchantPassword' => 'xxxxxx',
        }
    );
    return 1;
}

sub testSerializeRefUnitUndef {
    my $Config  = _loadPortalConfig();
    my $Product = _loadProduct('de_3203104010');
    
    my $aOffers = $Config->addProducts( [$Product] );
    is( scalar @{$aOffers}, 1, 'one created offer' );

    my $Request = DE_EPAGES::MeinPaket::API::Object::RequestAdapter::UploadRequest($Config, $aOffers, ['Stock']);

    isa_ok($Request, 'DE_EPAGES::MeinPaket::API::DTO::UploadRequest', 'Request');
    my $Offer = $Request->offers()->popProductOffer();
    isa_ok($Offer, 'DE_EPAGES::MeinPaket::API::DTO::ProductOffer', 'Offer');
    is($Offer->price(), '99.95', 'listprice = 99.95');
    my $UnitPrice = $Offer->unitprice();
    ok(!defined($UnitPrice), 'no unitprice');
}

sub testSerializeRefUnitPiece {
    my $Config  = _loadPortalConfig();
    my $Product = _loadProduct('de_3206199010');
    
    my $aOffers = $Config->addProducts( [$Product] );
    is( scalar @{$aOffers}, 1, 'one created offer' );

    my $Request = DE_EPAGES::MeinPaket::API::Object::RequestAdapter::UploadRequest($Config, $aOffers, ['Stock']);

    isa_ok($Request, 'DE_EPAGES::MeinPaket::API::DTO::UploadRequest', 'Request');
    my $Offer = $Request->offers()->popProductOffer();
    isa_ok($Offer, 'DE_EPAGES::MeinPaket::API::DTO::ProductOffer', 'Offer');
    is($Offer->price(), '26.95', 'listprice = 26.95');
    my $UnitPrice = $Offer->unitprice();
    isa_ok($UnitPrice, 'DE_EPAGES::MeinPaket::API::DTO::UnitPrice', 'UnitPrice');
    is($UnitPrice->unit(), 'per_piece', 'per_piece');
    is($UnitPrice->price(), '13.48', 'price = 13.48');
}


sub testSerializeRefUnitMililitre {
    my $Config  = _loadPortalConfig();
    my $Product = _loadProduct('ht_0801099030');
    
    my $aOffers = $Config->addProducts( [$Product] );
    is( scalar @{$aOffers}, 1, 'one created offer' );

    my $Request = DE_EPAGES::MeinPaket::API::Object::RequestAdapter::UploadRequest($Config, $aOffers, ['Stock']);

    isa_ok($Request, 'DE_EPAGES::MeinPaket::API::DTO::UploadRequest', 'Request');
    my $Offer = $Request->offers()->popProductOffer();
    isa_ok($Offer, 'DE_EPAGES::MeinPaket::API::DTO::ProductOffer', 'Offer');
    is($Offer->price(), '14.95', 'listprice = 14.95');
    my $UnitPrice = $Offer->unitprice();
    isa_ok($UnitPrice, 'DE_EPAGES::MeinPaket::API::DTO::UnitPrice', 'UnitPrice');
    is($UnitPrice->unit(), 'per_100ml', '100ml');
    is($UnitPrice->price(), '4.98', 'price = 4.98');
}

sub _loadProduct {
    my ($Alias) = @_;
    my $Product = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopMeinPaket', 'Products', $Alias ] ) );
    isa_ok( $Product, 'DE_EPAGES::Product::API::Object::Product' );
    return $Product;
}

sub _loadPortalConfig {
    my $Config = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopMeinPaket', 'PortalConfigs', 'MeinPaketTestConfig' ] ) );
    isa_ok( $Config, 'DE_EPAGES::MeinPaket::API::Object::MeinPaketConfig' );
    return $Config;
}

RunScript( Sub => \&Main );

1;
