use Test::More tests => 42;
use strict;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath ExistsObjectByPath ExistsObject);
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Order::API::Order qw ( CreateOrder CreateOrderWithContainer CreateOrderWithoutContainer );
use DE_EPAGES::Order::API::OrderDocument qw ( CreateOrderDocument );
use DE_EPAGES::Order::API::SalesVolume qw (GetSalesVolume);


my $ROOTPATH = '/Test';

## no critic qw( ProhibitLongLines )
sub Main {
    Init();

    my $Shop = LoadObjectByPath($ROOTPATH)->child('TestShop1');
    ok( $Shop->id, 'create shop ' . $Shop->id );

    my $CustomerNo = GetNextCustomerNo( $Shop );
    is( $CustomerNo, '1000', 'customer no. starts with 1000');

    my $CustomerFolder = GetCustomerFolder( $Shop );
    ok( !$CustomerFolder->existsChild( $CustomerNo ), 'customer no. does not exists yet');

    my $Customer = $Shop->get('CustomerClass')->insertObject({
        Parent => $CustomerFolder,
        Alias => $CustomerNo,
    });
    ok( $Customer->id, 'insert new customer' );
    ok( $Customer->alias eq $CustomerNo, 'Customer.Alias' );

    my $OrderNo = GetNextOrderNo( $Shop );
    is( $OrderNo, '1001', 'order no. starts with 1001');

    my $Order = $Customer->folder('Orders')->insertChild( 'CustomerOrder', {
        'Alias'     => $OrderNo,
        'Shop'      => $Shop,
        'Customer'  => $Customer,
    } );
    ok( $Order->id, 'insert new order' );
    ok( $Order->alias eq $OrderNo, 'Order.Alias' );
    ok( !defined $Order->get('LastStatus'), 'new order has no status' );

    # add a line item container
    my $Container = $Order->insertChild('LineItemContainer', {
        'Alias'             => 'LineItemContainer',
        'LanguageID'        => $Shop->get('LanguageID'),
        'LocaleID'          => $Shop->get('LocaleID'),
        'CurrencyID'        => $Shop->get('CurrencyID'),
        'TaxArea'           => $Shop->get('TaxMatrix')->defaultTaxArea,
        'TaxModel'          => $Shop->get('TaxModel'),
    });
    $Order->set({'LineItemContainer' => $Container});

    # test the sales volume
    $Container->set({'TotalBeforeTax' => 100.00});
    my $CurrencyID = $Shop->get('CurrencyID');

    my $TimeNow = GetCurrentDateTime();
    my $StartToday = $TimeNow->clone->truncate( to => 'day' );

    my $Tomorrow = $TimeNow->clone->add( 'days' => 1 );
    my $StartTomorrow = $Tomorrow->clone->add( 'days' => 1 );

    my ($SalesVolume,$SalesVolumeCount) = GetSalesVolume($Shop, $StartToday, $TimeNow, $CurrencyID);
    cmp_ok( $SalesVolume,'==', 100.00, 'Order found; SalesVolume for Order == 100.00' ); ## no critic qw(ProhibitMagicNumbers)
    cmp_ok( $SalesVolumeCount,'==', 1, 'Order found; SalesVolumeCount for Order == 1' ); ## no critic qw(ProhibitMagicNumbers)

    ($SalesVolume,$SalesVolumeCount) = GetSalesVolume($Shop, $StartTomorrow, $Tomorrow, $CurrencyID);
    cmp_ok( $SalesVolume,'==', 0, 'no Order found; SalesVolume == 0' );
    cmp_ok( $SalesVolumeCount,'==', 0, 'Order found; SalesVolumeCount for Order == 0' );

    _TestOrderDocuments( $Order );

    # Test set status
    $Order->set({'ReadyForShippingOn' => GetCurrentDateTime()});
    ok(defined $Order->get('ReadyForShippingOn'), 'Status ReadyForShippingOn updated');

    # test LastStatus
    my $Now = GetCurrentDateTime();
    $Order->set({'ViewedOn' => $Now->clone->add( days => 1 )});
    ok( $Order->get('LastStatus')->alias eq 'ViewedOn', 'Set ViewdOn' );

    $Order->set({'CancelledOn' => $Now->clone->add( days => 2 )});
    ok( $Order->get('LastStatus')->alias eq 'CancelledOn', 'Set CancelledOn' );

    my $BillingAddress = $Customer->folder('Addresses')->insertChild('Address', {
        Alias => 'BillingAddress',
        FirstName => 'Test',
        LastName => 'Tester',
    });
    $Customer->set({ 'BillingAddress', $BillingAddress });

    _TestCreateOrder( $Customer );
    _TestCreateOrderWithoutContainer( $Customer );
    _TestCreateOrderWithContainer( $Customer );

    my $OrderID = $Order->id;
    $Customer->delete;
    ok( !ExistsObject($OrderID), 'deleting the customer also deletes all orders' );

    Cleanup();
}

sub _TestOrderDocuments {
    my ($Order) = @_;

    # allow features for order documents
    my $Shop = $Order->get('Shop');
    $Shop->featureMaxValue( LoadObjectByPath('/Features/OrderDocument'), 5 );

    # Creation date from form
    my $DeliveryDate =  GetCurrentDateTime();
    my $IssueDate =     GetCurrentDateTime();

    my $Invoice = CreateOrderDocument( $Order, 'Invoice', 'Invoices', {
        'DeliveryDate' => $DeliveryDate,
        'IssueDate' => $IssueDate
    });

    my $NewInvoice = $Order->child('Invoices')->children->[0];
    is( $NewInvoice->id, $Invoice->id, 'first invoice successfully created with alias:'.$NewInvoice->alias);
    is( $Invoice->alias, '1001', 'alias of first invoice');

    my $PackingSlip = CreateOrderDocument( $Order, 'PackingSlip', 'PackingSlips', {
        'DeliveryDate' => $DeliveryDate,
        'IssueDate' => $IssueDate
    });
    my $NewPackingSlip = $Order->child('PackingSlips')->children->[0];
    is( $NewPackingSlip->id, $PackingSlip->id, 'first packing slip successfully created with alias:'.$NewPackingSlip->alias);
    is( $PackingSlip->alias, '1001', 'alias of first packing slip' );
}

sub _TestCreateOrder {
    my ($Customer) = @_;

    eval { CreateOrder( $Customer ); };
    ok( ExistsError() && GetError()->code eq 'NoShipping', 'Can\'t test CreateOrder() if the shop has no shipping methods' );

    my $Shop = $Customer->get('Shop');
    is( $Shop->get('CurrencyID'), 'GBP', 'Shop.Currency' );

    SKIP: {
        my $SkipTests = 9;
        skip( 'no shipping method available', $SkipTests ) unless ExistsObjectByPath('/ShippingTypes/Free');
        skip( 'no payment method available', $SkipTests ) unless ExistsObjectByPath('/PaymentTypes/Free');
        skip( 'no product class', $SkipTests ) unless $Shop->class->existsAttribute('ProductClass');

        # add a product, shipping and payment method to the shop
        $Shop->featureMaxValue('Products', 1);
        my $Product = $Shop->get('ProductFolder')->insertChild( $Shop->get('ProductClass'), {
            'Alias' => '08/15'
        });
        $Product->setListPrice('GBP', 0, 10.0); ## no critic qw(ProhibitMagicNumbers)

        $Shop->featureMaxValue('PaymentMethods', 1);
        my $PaymentMethod = LoadObjectByPath('/PaymentTypes/Free')->newPaymentMethod($Shop, {
            'Alias' => 'FreePayment',
            'IsActivated' => 1,
            'IsDefault' => 1,
        });

        $Shop->featureMaxValue('ShippingMethods', 1);
        my $ShippingMethod = LoadObjectByPath('/ShippingTypes/Free')->newShippingMethod($Shop, {
            'Alias' => 'FreeShipping',
            'IsActivated' => 1,
            'IsDefault' => 1,
        });

        # create an order
        my $Order = CreateOrder( $Customer );
        is( $Order->get('Customer')->id, $Customer->id, 'CreateOrder: Order.Customer' );
        ok( defined $Order->get('ShopAddress'), 'CreateOrder: shop address exists' );

        my $Container = $Order->get('LineItemContainer');
        is( $Container->get('Alias'), 'LineItemContainer', 'CreateOrder: Order.LineItemContainer' );
        is( $Container->get('LocaleID'), $Customer->getSite->get('LocaleID'), 'CreateOrder: LineItemContainer.LocaleID' );
        ok( !defined $Container->get('Shipping'), 'CreateOrder: there is no shipping method, because there are no items that require shipping' );
        ok( defined $Container->get('Payment'), 'CreateOrder: payment exists' );
        is( $Container->get('Payment')->get('PaymentMethod')->id, $PaymentMethod->id, 'CreateOrder: LineItemContainer.Payment' );

        # add a product that requires shipping
        $Product->addToContainer( $Container, { Quantity => 2 } );
        ok( defined $Container->get('Shipping'), 'CreateOrder: now we use the default shipping method' );
        is( $Container->get('Shipping')->get('ShippingMethod')->id, $ShippingMethod->id, 'CreateOrder: LineItemContainer.Shipping' );
        ok( defined $Container->get('Payment'), 'CreateOrder: Order.Payment' );
    }
}

sub _TestCreateOrderWithContainer {
    my ($Customer) = @_;

    my $Order = CreateOrderWithContainer( $Customer );
    is( $Order->get('Customer')->id, $Customer->id, 'CreateOrderWithContainer: Order.Customer' );
    ok( defined $Order->get('ShopAddress'), 'CreateOrderWithContainer: shop address exists' );

    my $Container = $Order->get('LineItemContainer');
    is( $Container->get('Alias'), 'LineItemContainer', 'CreateOrderWithContainer: Order.LineItemContainer' );
    is( $Container->get('LocaleID'), $Customer->getSite->get('LocaleID'), 'CreateOrderWithContainer: Order.LineItemContainer.LocaleID' );
    ok( !defined $Container->get('Shipping'), 'CreateOrderWithContainer: no shipping' );
    ok( !defined $Container->get('Payment'), 'CreateOrderWithContainer: no payment' );
}

sub _TestCreateOrderWithoutContainer {
    my ($Customer) = @_;

    my $Order = CreateOrderWithoutContainer( $Customer );
    is( $Order->get('Customer')->id, $Customer->id, 'CreateOrderWithoutContainer: Order.Customer' );
    ok( !defined $Order->get('ShopAddress'), 'CreateOrderWithoutContainer: shop address exists' );
    ok( !defined $Order->get('LineItemContainer'), 'CreateOrderWithoutContainer: no LineItemContainer' );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
