use Test::More tests => 87;
use warnings;
use strict;
use utf8;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByPath );
use DE_EPAGES::Presentation::API::Session qw( InsertSession SetSessionAttribute GetSessionAttribute );
use DE_EPAGES::Presentation::API::Table::Session qw( GetInfoSessionBySessionID DeleteSession ExistsSession);
use DE_EPAGES::Presentation::API::Table::SessionAttribute qw( DeleteSessionAttribute ExistsSessionAttribute );
use DE_EPAGES::Presentation::API::Table::SessionParameter qw( ExistsSessionParameter );

sub Main {

    # fake
    local *DE_EPAGES::Presentation::API::Session::GetAppServer = sub {
        return Mock::AppServer->new;
    };

    ## no critic qw( ProhibitMagicNumbers )
    # insert a session
    my $User = LoadObjectByPath('/Users/wsadmin');
    my $SessionID = InsertSession( $User, 1 );
    is( length $SessionID, 32, 'SessionID length' );

    my $hSession = GetInfoSessionBySessionID( $SessionID );
    is( length $hSession->{'Token'}, 10, 'Token length' );

    my $IntSessionID = $hSession->{'IntSessionID'};
    ok( $IntSessionID =~ /^[1-9]\d*$/, 'IntSessionID is integer' );

    my @TestCases = (
        { 'action' => 'set some chars',   'value' => 'abcde', },
        { 'action' => 'set empty string', 'value' => '', },
        { 'action' => 'set undef',        'value' => undef, },
        { 'action' => 'set 499 chars',    'value' => 'a' x 499, },
        { 'action' => 'set 500 chars',    'value' => 'a' x 500, },
        { 'action' => 'set 501 chars',    'value' => 'a' x 501, },
        { 'action' => 'set 1010 chars',   'value' => 'abcde' x 202, },
        { 'action' => 'set 499 chars',    'value' => 'abcde' x 99 . 'fghi', },

        { 'action' => '(utf8) set some chars', 'value' => 'äöüÄÖÜßáéíóúàèìòù', },
        { 'action' => '(utf8) set 499 chars',  'value' => 'ä' x 499, },
        { 'action' => '(utf8) set 500 chars',  'value' => 'ö' x 500, },
        { 'action' => '(utf8) set 501 chars',  'value' => 'ü' x 501, },
        { 'action' => '(utf8) set 1010 chars', 'value' => 'äöüÄÖÜßáéí' x 101, },
        { 'action' => '(utf8) set 499 chars',  'value' => 'äöüÄÖÜßáéí' x 99 . 'òùòù', },

        { 'action' => 'set 254 chars', 'value' => 'abcde',
          'attribute_name' => 'TEST_SessionAttribute_' . 'a' x 232, },

        { 'action' => 'set 255 chars', 'value' => 'abcde',
          'attribute_name' => 'TEST_SessionAttribute_' . 'a' x 233, },

        { 'action' => '(utf8) set 254 chars', 'value' => 'abcde',
          'attribute_name' => 'TEST_SessionAttribute_' . 'ä' x 232, },

        { 'action' => '(utf8) set 255 chars', 'value' => 'abcde',
          'attribute_name' => 'TEST_SessionAttribute_' . 'ä' x 233, },
    );

    # Init vars.
    my $dbi = GetCurrentDBHandle();
    my $IntSessionAttributeID = undef;
    my $DefaultAttributeName = 'TEST_SessionAttribute';
    my %AttributeIDs;

    # Process all test cases
    foreach my $hCase (@TestCases){
        my $Action = $hCase->{'action'};
        my $Value = $hCase->{'value'};
        my $AttributeName = exists $hCase->{'attribute_name'}
                            ? $hCase->{'attribute_name'}
                            : $DefaultAttributeName;

        # Set attribute value.
        eval { SetSessionAttribute( $IntSessionID, $AttributeName, $Value) };
        ok(!ExistsError(),"Set attribute: $Action: ");
        diag GetError()->message if ExistsError();

        # Get AttributeID if it is still unknown.
        unless(exists $AttributeIDs{$AttributeName}){
            my $aaRows = $dbi->execute( 'SELECT sessionattributeid FROM sessionattribute WHERE name = ?', [ $AttributeName ] );
            $IntSessionAttributeID = $aaRows->[0][0];
            $AttributeIDs{$AttributeName} = $IntSessionAttributeID;
        }

        ## The Value should be stored in table sessionparameter
        my $aaRows = $dbi->execute('SELECT value FROM sessionparameter WHERE intsessionid = ? AND sessionattributeid = ?', [ $IntSessionID, $IntSessionAttributeID ] );
        my $DBResult = $aaRows->[0][0];
        is( $DBResult, $Value, "Attribute at table 'sessionparameter': $Action" );

        # API should return the same value that has been set before.
        my $APIResult = eval { GetSessionAttribute( $IntSessionID, $AttributeName ) };
        ok(!ExistsError(),"Get attribute: $Action");
        is( $APIResult, $Value, "Attribute from API: $Action" );
    }

    # Delete the attributes and test if they are really gone.
    foreach my $AttributeID (values %AttributeIDs){
        if(ExistsSessionAttribute($AttributeID)){
            DeleteSessionAttribute($AttributeID);
            ok(!ExistsSessionAttribute($AttributeID), 'Delete attribute: ' . $AttributeID);
            ok(!ExistsSessionParameter($IntSessionID, $AttributeID), 'Delete parameter of: ' . $AttributeID);
        }
    }

    DeleteSession($IntSessionID);
    ok(!ExistsSession($IntSessionID), 'Delete session.');

    my $SQL =<<'EOC';
SELECT sp.sessionattributeid AS id, sp.sessionattributeid AS id
FROM sessionparameter as sp
WHERE sp.intsessionid = ?
EOC
    my $aaRows = $dbi->execute($SQL , [ $IntSessionID ] );
    my $DBResult = $aaRows->[0][0];
    ok(!$DBResult, 'Delete session parameters.' );
}

RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

## no critic qw( RequireCommentForExportedFunctions )
package Mock::AppServer;

sub new {
    my $class = shift;
    return bless {}, $class;
}

sub address {
    return '0.0.0.0';
}

sub port {
    return 0;
}

sub serverVariables {
    return {
        HTTP_REFERER => 'x',
        REMOTE_ADDR => 'ip',
        REMOTE_HOST => 'host',
        REMOTE_USER => 'user',
    };
}
