use strict;
use Test::More tests => 9;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (JoinObjectPath ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

## no critic qw( ProhibitMagicNumbers )

my $ROOTPATH = '/Test';

sub Main {
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    my $Shop = LoadObjectByPath('/Test/TestShop');
    $Shop->set({'DomainName' => 'www.myshop.lan'});

    my $Product = LoadObjectByPath('/Test/TestShop/Products/MyPetrol');
    $Product->set({
        'Name' => 'My petrol product!',
        'URI' => 'my-petrol-product',
    }, $Shop->get('LanguageID'));

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

    # product canonical
    my $ProductCanonical = 'http://www.myshop.lan/my-petrol-product';
    my $Response = $FakeRequest->get($ProductCanonical);
    cmp_ok( $Response->code, '==', 200, 'Product page via shorturl' );
    my $Content = $Response->decoded_content;
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$ProductCanonical\E\"\s+\/>/xs, 'Canonical tag ok');

    # variation product canonical
    my $SuperProduct = LoadObjectByPath('/Test/TestShop/Products/Shirt');
    $SuperProduct->set({
        'Name' => 'My shirt product!',
        'URI' => 'my-shirt-product',
    }, $Shop->get('LanguageID'));
    my $SubProduct = LoadObjectByPath(JoinObjectPath(['Test', 'TestShop', 'Products', 'Shirt', 'SubProducts', 'Shirt blue']));

    $Response = $FakeRequest->get($SubProduct->get('WebUrl'));
    cmp_ok( $Response->code, '==', 200, 'Sub product page via weburl' );
    $Content = $Response->decoded_content;
    my $SuperProductCanonical = 'http://www.myshop.lan/my-shirt-product';
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$SuperProductCanonical\E\"\s+\/>/xs,
                    'Canonical tag points to super product');

    # mobile product canonical
    my $aHeaders = [
        'User-Agent' => 'Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 ' .
                        '(KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16',
    ];

    # without mobile feature the request goes to the normal product sf page
    $ProductCanonical = 'http://www.myshop.lan/my-petrol-product';
    $Response = $FakeRequest->get($ProductCanonical, $aHeaders);
    cmp_ok( $Response->code, '==', 200, 'Without mobile feature product page with mobile devide shows sf page' );

    $Shop->featureMaxValue('MobileView', 1);

    $ProductCanonical = 'http://www.myshop.lan/my-petrol-product';
    $Response = $FakeRequest->get($ProductCanonical, $aHeaders);
    cmp_ok( $Response->code, '==', 301, 'Mobile product page via shorturl redirects' );

    $Response = $FakeRequest->get('http://www.myshop.lan'.$Response->header('Location'), $aHeaders);
    $Content = $Response->decoded_content;
    ok($Content =~ /<link\s+rel=\"canonical\"\s+href=\"\Q$ProductCanonical\E\"\s+\/>/xs, 'Canonical tag ok');
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    Main();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});
