use strict;
use Test::More tests => 9;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::SEO::API::SEO qw ( GetSEOCheckValues );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);

my $ROOTPATH = '/Test';

## no critic qw( ProhibitLongLines ProhibitMagicNumbers )

sub Main {
    my $Shop = LoadObjectByPath('/Test/TestShop');

    my $LangID = $Shop->get('LanguageID');

    # general attributes
    $Shop->set({ 'DomainName' => 'www.mytestshop.com' });
    $Shop->set({ 'Name' => 'MyTestShop' }, $LangID);
    my $hValues = GetSEOCheckValues($Shop, $Shop->get('Categories'), $LangID);
    my $hGeneralValues = {
        FormerSEOQuality => 0,
        SiteUrl => $Shop->get('WebUrl'),
        Domain => $Shop->get('DomainName'),
    };

    # use product and set some SEO-nice values
    my $Product = LoadObjectByPath('/Test/TestShop/Products/Cap');
    $Product->set({
        'Name' => 'Nice test article - all u ever wanted to know!',
        'Description' => 'Curabitur pretium elementum purus, ac tincidunt tellus lobortis id. In quis blandit purus. Quisque quis elit varius est faucibus.',
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. <div><img src="meins.jpg" width="100" height="100" alt="" /></div> Morbi dignissim lacus eu ipsum placerat sit amet volutpat leo vestibulum. <ul><li>One</li><li>Two</li></ul>',
        'URI' => 'top-nice-cool-cap',
    }, $LangID);
    TriggerHook('SEOCheckTests', {
        'Object'        => $Product,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    my $hLangValues = GetSEOCheckValues($Shop, $Product, $LangID);

    $hValues->{$LangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };
    foreach my $Token ( qw (Url Title Description)) {
        cmp_ok($hValues->{$LangID}->{$Token}->{'Rating'}, '==', 1, 'GetSEOCheckValues Rating 1');
    }
    cmp_ok($hValues->{$LangID}->{'AdvDescription'}->{'Rating'}, '==', 3, 'GetSEOCheckValue AdvDescription Rating orange (missing alt attr)');


    # alt tag now present
    $Product->set({
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. <div><img src="meins.jpg" width="100" height="100" alt="My cool image" /></div> Morbi dignissim lacus eu ipsum placerat sit amet volutpat leo vestibulum. <ul><li>One</li><li>Two</li></ul>',
    }, $LangID);
    TriggerHook('SEOCheckTests', {
        'Object'        => $Product,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    $hLangValues = GetSEOCheckValues($Shop, $Product, $LangID);

    $hValues->{$LangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };

    cmp_ok($hValues->{$LangID}->{'AdvDescription'}->{'Rating'}, '==', 1, 'GetSEOCheckValue AdvDescription Rating 1');



    # now two images: one with alt tag present, one not
    $Product->set({
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. <div><img src="meins.jpg" width="100" height="100" alt="My cool image" /></div> Morbi dignissim lacus eu ipsum placerat <img src="meins2.jpg" width="120" height="150" alt="" /> sit amet volutpat leo vestibulum. <ul><li>One</li><li>Two</li></ul>',
    }, $LangID);
    TriggerHook('SEOCheckTests', {
        'Object'        => $Product,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    $hLangValues = GetSEOCheckValues($Shop, $Product, $LangID);
    $hValues->{$LangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };
    cmp_ok($hValues->{$LangID}->{'AdvDescription'}->{'Rating'}, '==', 3, 'GetSEOCheckValue AdvDescription Rating orange (missing alt attr)');


    # again two images: both with alt tag present
    $Product->set({
        'Text' => 'Nunc arcu <em>metus</em>, dapibus in bibendum et, aliquet eu lectus. Donec ipsum tellus, tempor non dignissim at, vehicula lacinia enim. <div><img src="meins.jpg" width="100" height="100" alt="My cool image" /></div> Morbi dignissim lacus eu ipsum placerat <img src="meins2.jpg" width="120" height="150" alt="My second cool image" /> sit amet volutpat leo vestibulum. <ul><li>One</li><li>Two</li></ul>',
    }, $LangID);
    TriggerHook('SEOCheckTests', {
        'Object'        => $Product,
        'Shop'          => $Shop,
        'Values'        => $hGeneralValues,
    });
    $hLangValues = GetSEOCheckValues($Shop, $Product, $LangID);
    $hValues->{$LangID} = {
        %$hGeneralValues,
        %$hLangValues,
    };
    cmp_ok($hValues->{$LangID}->{'AdvDescription'}->{'Rating'}, '==', 1, 'GetSEOCheckValue AdvDescription Rating 1');
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    Main();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});
