use Test::More;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File  qw ( GetFileContent WriteFile DeleteFile DeleteDirectory );
use DE_EPAGES::TLE::API::TLE    qw ( ReplaceTLEVars GetByteCodeByText );
use DE_EPAGES::TLE::API::BaseProcessor;
use DE_EPAGES::TLE::t::tlevars;

our $TEST_ONE;

sub Main {
    Initialize();
    if ( defined $TEST_ONE ) {
        plan tests => 1;
        ok ( Test_Package($TEST_ONE ), $TEST_ONE );
    }
    else {
        my @Files = glob '*_t.txt';
        plan tests => 1 + scalar @Files;

        foreach my $File ( @Files ) {
            if($File =~ /^(.+)_t\.txt$/) {
                my $TestCase = $1;
                ok ( Test_Package($TestCase ), $File );
            }
            else {
                die "invalid file name $File";
            }
        }

        Test_ReplaceTLEVars( '05_cond' );
    }
    Cleanup();
}

local $| = 1;

RunScript(
    'Sub' => \&Main
);

sub ErrorDefinition {
    return {
        NO_EQUAL_LINE => 'Output is different at line #AtLine'
    };
}

sub Test_ReplaceTLEVars {
    my ($i) = @_;

    my $rTemplate = GetFileContent( $i.'_t.txt' );
    my $byteCode = GetByteCodeByText( $rTemplate );
    my %Vars = (
        %{ DE_EPAGES::TLE::t::tlevars::GetLoops($i) },
        %{ DE_EPAGES::TLE::t::tlevars::GetVars($i) }
    );
    my $isContent = ReplaceTLEVars($byteCode, \%Vars );
    my $rExContent = GetFileContent($i.'_e.txt');
    eval { CompareContent($rExContent, \$isContent) };
    ok( !ExistsError(), 'ReplaceTLEVars' ) or diag GetError->message;
    DeleteFile($i.'_i.txt');
}

sub Test_Package {
    my ($i) = @_;

    my $isContent;
    eval {
        my $Loops =  DE_EPAGES::TLE::t::tlevars::GetLoops($i);
        my $Vars  =  DE_EPAGES::TLE::t::tlevars::GetVars($i);

        local $DE_EPAGES::TLE::API::Execute::TEMPLATE_SOURCE = $i.'_t.txt';
        $isContent = ProcessFileTemplate($i.'_t.txt', $Vars , $Loops);
        my $rExContent = GetFileContent($i.'_e.txt');
        CompareContent($rExContent, \$isContent);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $Error->addVars( { 'Test' => $i } );
        note $Error->longMessage();
        WriteFile($i.'_i.txt', \$isContent) if defined $isContent;
        return 0;
    } else {
        DeleteFile($i.'_i.txt');
    }
    return 1;
}

sub ProcessFileTemplate {
    my ( $FileName, $hVars, $hLoops ) = @_;

    my $rTemplate = GetFileContent( $FileName );

    my $byteCode = GetByteCodeByText( $rTemplate );
    my %TempVars = (%$hVars, %$hLoops);

    my $TLEProcessor = DE_EPAGES::TLE::API::BaseProcessor->new;
    DE_EPAGES::TLE::API::PrefixHandler->new->register( $TLEProcessor );

    return $TLEProcessor->replaceTLEs($byteCode, \%TempVars);
}

sub CompareContent {
    my ($rExpected, $rIs) = @_;

    my @aExpected = split /\n/, $$rExpected;
    my @aIs = split /\n/, $$rIs;
    if (scalar @aIs != scalar @aExpected) {
        diag('expected:'.$$rExpected);
        diag('is:'.$$rIs);
        Error('NO_EQUAL_LINES', {'CountLinesIs' => scalar @aIs, 'CountLinesExpected' => scalar @aExpected } );
    }
    for (my $i=0;$i < scalar @aExpected; $i++) {
        if ($aExpected[$i] ne $aIs[$i]) {
            Error('NO_EQUAL_LINE', {'AtLine' => $i + 1, 'Exp'=>$aExpected[$i], 'Ist'=> $aIs[$i] } );
        }
    }
    return 1;
}

sub Initialize {
    Cleanup();
}

sub Cleanup {
    DeleteDirectory( 'ctmpl', { recursive => 1 } );
}

1;

