use strict;
use Test::More tests => 15;
use HTTP::Status;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Net qw( GetHttpResponse );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath LoadRootObject);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ShopPath = '/Shops/DemoShop';

use constant MAX_HTML_SIZE => 25000; # 6.13.0-pre: 24119 (2011/12/08); 6.12.2-pre: 26634; 25273 (2011/06/22), 6.12.0-pre: 24495 bytes, 6.11.3: 25261 bytes;  6.11.0: 27906 bytes
use constant MAX_JS_SIZE => 340000;  # 6.13.0-pre: 335716 (2011/12/08); 6.12.0 (2011/05/31): 312956; 6.11.3: 302471 bytes; 6.11.0: 690253 bytes (CRLF)
use constant MAX_JS_COUNT => 2;
use constant MAX_CSS_SIZE => 300000; # 6.13.0-pre: 294848 (2011/12/20); 6.11.2: 264541 bytes; 6.11.0: 283019 Bytes (CRLF)
use constant MAX_CSS_COUNT => 5;
use constant MAX_TOTAL_SIZE => 661210; # 6.13.0-pre: 654735 (2011/12/20); 6.12.2-pre: 601480; 600620 (2011/06/22); 6.12.0 (2011/05/31): 589803 bytes; 6.11.3: 561906 bytes; 6.11.0: 1001178 bytes
use constant MAX_TOTAL_COUNT => 8;

#========================================================================================
# function     TimeThis
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Time = TimeThis( $Code );
# syntax       $Time = TimeThis( sub { sleep 3; } );
#----------------------------------------------------------------------------------------
# description  Returns the real time required to execute the code block.
#----------------------------------------------------------------------------------------
# input        $Code | code to execute | code reference
# return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

#========================================================================================
# function     SetShopAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetShopAttributes( $StoreName, $hValues );
#----------------------------------------------------------------------------------------
# description  Sets attributes on the DemoShop object.
#----------------------------------------------------------------------------------------
# input        $StoreName | store name | string
# input        $hValues | attribute values | ref.hash
#========================================================================================
sub SetShopAttributes {
    my ($StoreName, $hValues) = @_;

    RunOnStore(
        Store      => $StoreName,
        Sub        => sub {
            LoadObjectByPath($ShopPath)->set($hValues);
        }
    );
    return;
}

sub TestExternalIncludes {
    my ($FakeRequest, $ShopURL) = @_;

    my $Response = $FakeRequest->get( $ShopURL );
    die "Can't get $ShopURL: ".$Response->status_line unless $Response->is_success;

    my $Content = $Response->content;

    # filter debug info
    $Content =~ s/\n.*?BEGIN INCLUDE.*?\n//g;
    $Content =~ s/\n.*?END INCLUDE.*?\n//g;
    $Content =~ s/\n.*?END OF PAGE.*?\n//g;
    $Content =~ s/\n.*?CachedInclude.*?\n//g;

#   use DE_EPAGES::Core::API::File qw( WriteFile );
#   WriteFile('content.html', \$Content, { binary => 1 });

    my $ContentLength = length($Content);
    cmp_ok( $ContentLength, '<', MAX_HTML_SIZE, "HTML code size (actual: $ContentLength)" );

    my $hCSSLinks = _ExtractExternalStyles( $ShopURL, $Content );
    my $hJSLinks = _ExtractExternalScripts( $ShopURL, $Content );
    cmp_ok( $hCSSLinks->{'Count'}, '<=', MAX_CSS_COUNT, "Number of CSS files (actual: $hCSSLinks->{'Count'})" );
    cmp_ok( $hCSSLinks->{'Size'}, '<', MAX_CSS_SIZE, "CSS size (actual: $hCSSLinks->{'Size'})" );
    cmp_ok( $hJSLinks->{'Count'}, '<=', MAX_JS_COUNT, "Number of JS files (actual: $hJSLinks->{'Count'})" );
    cmp_ok( $hJSLinks->{'Size'}, '<', MAX_JS_SIZE, "JS size (actual: $hJSLinks->{'Size'})" );

    my $TotalCount = 1 + $hCSSLinks->{'Count'} + $hJSLinks->{'Count'};
    my $TotalSize = $ContentLength + $hJSLinks->{'Size'} + $hCSSLinks->{'Size'};
    cmp_ok( $TotalCount, '<=', MAX_TOTAL_COUNT, "Total number of files (actual: $TotalCount)" );
    cmp_ok( $TotalSize, '<=', MAX_TOTAL_SIZE, "Total page size (actual: $TotalSize)" );

    use Data::Dumper;
    if( $TotalCount > MAX_TOTAL_COUNT || $TotalSize > MAX_TOTAL_SIZE ) {
        diag Dumper( $hCSSLinks, $hJSLinks );
    }
}

sub _ExtractExternalScripts {
    my ($ShopURL, $Content) = @_;

    my $BaseURI = URI->new($ShopURL);

    my $Count = 0;
    my $Size = 0;
    my @Links;
    foreach my $hAttributes( grep { exists $_->{'src'} } _AttributesByTagName($Content, 'script') ) {
        my $ScriptURL = URI->new( $hAttributes->{'src'} )->abs( $BaseURI )->as_string;
        my $Response = GetHttpResponse( $ScriptURL );
        die "Can't get script '$ScriptURL': " . $Response->status_line unless $Response->is_success;

        $Count++;
        $Size += length( $Response->content );
        push @Links, {
            URL => $hAttributes->{'src'},
            Size => length( $Response->content ),
        };
    }

    return {
        Count => $Count,
        Size => $Size,
        Links => \@Links,
    };
}

sub _ExtractExternalStyles {
    my ($ShopURL, $Content) = @_;

    my $BaseURI = URI->new( $ShopURL );
    my @StyleLinks = map { $_->{'href'} } grep { $_->{'rel'} =~ /^stylesheet$/i } _AttributesByTagName($Content, 'link');

    my $Count = 0;
    my $Size = 0;
    my @Links;
    foreach my $StyleLink( @StyleLinks ) {
        my $StyleURL = URI->new( $StyleLink )->abs( $BaseURI )->as_string;
        my $hStyle = _ExtractImportStyles( $StyleURL );
        $Count += $hStyle->{'Count'};
        $Size += $hStyle->{'Size'};
        push @Links, @{ $hStyle->{'Links'} };
    }

    foreach my $StyleLink ( map { $_->{'URL'} } map { _ParseImports( $_ ) } _TextsByTagName( $Content, 'style' ) ) {
        my $StyleURL = URI->new( $StyleLink )->abs( $BaseURI )->as_string;
        my $hStyle = _ExtractImportStyles( $StyleURL );
        $Count += $hStyle->{'Count'};
        $Size += $hStyle->{'Size'};
        push @Links, @{ $hStyle->{'Links'} };
    }

    return {
        Count => $Count,
        Size => $Size,
        Links => \@Links,
    };
}

sub _ExtractImportStyles {
    my ($StyleURL) = @_;

    my $Response = GetHttpResponse( $StyleURL );
    die "Can't get style '$StyleURL': " . $Response->status_line unless $Response->is_success;
    my $Content = $Response->content;

    my $BaseURI = URI->new( $StyleURL );
    my $Count = 1;
    my $Size = length $Content;
    my @Links = ( {
        URL => $StyleURL,
        Size => length $Content,
    } );

    foreach my $StyleLink ( map { $_->{'URL'} } _ParseImports( $Content ) ) {
        my $ImportStyleURL = URI->new( $StyleLink )->abs( $BaseURI )->as_string;
        my $hStyle = _ExtractImportStyles( $ImportStyleURL );
        $Count += $hStyle->{'Count'};
        $Size += $hStyle->{'Size'};
        push @Links, @{ $hStyle->{'Links'} }
    }

    return {
        Count => $Count,
        Size => $Size,
        Links => \@Links,
    };
}

sub _ParseImports {
    my ($Content) = @_;

    # eg. @import url( 'import4.css' ) tv, print;
    my @Imports;
    $Content =~ s/
        \@import
        (?:\s+url\s*\()? # open (
        \s*
        (["']?)    # open quote (optional) "
        (.*?)      # $2 = relative file name
        \1         # close quote
        \s*
        \)?        # close )
        \s*
        (.*?)      # $3 = media types
        \s*
        ;
    /push @Imports, { URL => $2, MediaTypes => $3};/xges;

    return @Imports;
}

sub _ParseAttributes {
    my ($Attributes) = @_;

    my %Attributes;
    while( $Attributes =~ /(\w+)\s*=\s*["'](.*?)["']/sgx ) { # "
        $Attributes{$1} = $2;
    }

    return \%Attributes;
}

sub _AttributesByTagName {
    my ($Content, $TagName) = @_;

    my @Tags;
    while( $Content =~ m{
        <
        \Q$TagName\E\s+     # tag name
        (.*?)               # attributes string
        (?:/)?             # optional /
        >
    }isgx
    ) {
        push @Tags, _ParseAttributes( $1 );
    }

    return @Tags;
}

sub _TextsByTagName {
    my ($Content, $TagName) = @_;

    my @Texts;
    while( $Content =~ m{
        <\Q$TagName\E\b.*?>     # open tag
        (.*)                    # text
        <\/\Q$TagName\E>        # close tag
    }isgx
    ) {
        push @Texts, $1;
    }

    return @Texts;
}

sub Main {

    my $StoreName = GetTestSite();
    my $DemoShopURL;
    my $JSDebugLevel;

    RunOnStore(
        Store      => $StoreName,
        Sub        => sub {
            if( ExistsObjectByPath($ShopPath) ) {
                my $DemoShop = LoadObjectByPath($ShopPath);
                $DemoShop->set({
                    IsClosed => 0,
                    ClosedByMerchant => 0,
                    ClosedByProvider => 0,
                });
                $DemoShopURL = $DemoShop->get('WebUrl');
            }
            $JSDebugLevel = LoadRootObject()->get('JSDebugLevel');
        }
    );

    SKIP: {
        skip 'no DemoShop installed', 3 unless defined $DemoShopURL;

        is( $JSDebugLevel,0 , "System.JSDebugLevel" );

        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

        my $Sub = sub {
            my $Response = $FakeRequest->get( $DemoShopURL );
            if( !$Response->is_success ) {
                die $Response->status_line;
            }
        };

        SKIP: {
            skip( 'Forced skip of timing tests.', 3 ) if $ENV{'SKIP_TIMING'};

            my $FirstTime = TimeThis( $Sub );
            cmp_ok( $FirstTime, '<', 4, sprintf('start page: first call: %0.3f s', $FirstTime) ); ## no critic qw(ProhibitMagicNumbers)

            my $SecondTime = TimeThis( $Sub );
            cmp_ok( $SecondTime, '<', 0.8, sprintf('start page: second call: %0.3f s', $SecondTime) ); ## no critic qw(ProhibitMagicNumbers)

            my $ThirdTime = TimeThis( $Sub );
            cmp_ok( $ThirdTime, '<', 0.8, sprintf('start page: third call: %0.3f s', $ThirdTime) ); ## no critic qw(ProhibitMagicNumbers)
        }

        SetShopAttributes( $StoreName,{
            'IsClosed' => 1,
        });

        is( $FakeRequest->get( $DemoShopURL )->code, RC_SERVICE_UNAVAILABLE, 'status code Site.IsClosed' );

        SetShopAttributes( $StoreName,{
            'IsClosed' => 0,
            'ClosedByMerchant' => 1,
        });

        is( $FakeRequest->get( $DemoShopURL )->code, RC_OK, 'status code Site.ClosedByMerchant' );

        SetShopAttributes( $StoreName,{
            'ClosedByMerchant' => 0,
            'ClosedByProvider' => 1,
        });

        is( $FakeRequest->get( $DemoShopURL )->code, RC_SERVICE_UNAVAILABLE, 'status code Site.ClosedByProvider' );

        SetShopAttributes( $StoreName,{
            'ClosedByProvider' => 0,
        });

        is( $FakeRequest->get( $DemoShopURL )->code, RC_OK, 'status code (not closed)' );

        TestExternalIncludes( $FakeRequest, $DemoShopURL );
    };
    return;
}

RunScript( Sub => \&Main, Debug => 1 );
