use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( BlockWarnings );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::MeinPaket::API::Order qw ( ImportMeinPaketOrders );
use DE_EPAGES::MeinPaket::API::Table::MeinPaketCustomerOrderMap qw (
    DeleteMeinPaketCustomerOrderMap
    GetPKeyMeinPaketCustomerOrderMapsByMeinPaketOrderID
);
use DE_EPAGES::MeinPaket::API::XML::ResponseHandler;
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    JoinObjectPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::Language qw ( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::MeinPaket::t::TestHelper qw ( RunWithRemoteCall );
use Readonly;
use Test::More;

Readonly my $ROOTPATH     => '/Test';
Readonly my $XMLDIR => $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/MeinPaket/t/';

## no critic (ProhibitMagicNumbers ProhibitLongLines ProhibitFunctionSize)
sub testImportOrders {

    # parse xml
    my $Result
      = DE_EPAGES::MeinPaket::API::XML::ResponseHandler->importXML( ${ GetFileContent( $XMLDIR . 'testImportOrders.xml', { 'encoding' => 'utf-8' } ) } );
    isa_ok( $Result, 'DE_EPAGES::MeinPaket::API::DTO::QueryResponse', 'Result' );
    is( scalar @{ $Result->OrderList() }, 2, 'Orders' );

    # import orders
    my $PortalConfig = _loadPortalConfig();
    my $aOrders = ImportMeinPaketOrders( $PortalConfig, $Result );
    is( scalar @{$aOrders}, '2', 'Two orders' );

    # some preparation
    my $LanguageID = GetPKeyLanguageByCode('de');

    # check first order - without product assignment
    my $RemoteOrder = $Result->OrderList()->[0];
    my $Order       = $aOrders->[0];
    my $Container   = $Order->get('LineItemContainer');

    # general structure
    isa_ok( $RemoteOrder,                   'DE_EPAGES::MeinPaket::API::DTO::Order',              'RemoteOrder' );
    isa_ok( $Order,                         'DE_EPAGES::Order::API::Object::CustomerOrder',       'Order1' );
    isa_ok( $Order->get('MeinPaketConfig'), 'DE_EPAGES::MeinPaket::API::Object::MeinPaketConfig', 'Order1.MeinPaketConfig' );
    like( $Container->get('Shipping')->get( 'NameOrAlias', $LanguageID ), qr{Standard}, 'Order1.Shipping "Standard"' );
    cmp_ok( $Container->get('Shipping')->get('LineItemPrice'), '==', 0.0, 'Order1.Shipping.LineItemPrice' );
    is( $Container->get('Payment')->get( 'NameOrAlias', $LanguageID ), 'MeinPaket.de', 'Order1.Payment "MeinPaket.de"' );

    # billing address
    my $BillingAddress = $Order->get('BillingAddress');
    isa_ok( $BillingAddress, 'DE_EPAGES::Address::API::Object::Address', 'Order1.BillingAddress' );
    is( $BillingAddress->get('FirstName'),          'Tom',          'Order1.BillingAddress.FirstName' );
    is( $BillingAddress->get('LastName'),           'Tester',       'Order1.BillingAddress.LastName' );
    is( $BillingAddress->get('DisplayName'),        'Tom Tester',   'Order1.BillingAddress.DisplayName' );
    is( $BillingAddress->get('Street'),             'Hansaring 44', 'Order1.BillingAddress.Street' );
    is( $BillingAddress->get('Zipcode'),            '50670',        'Order1.BillingAddress.ZipCode' );
    is( $BillingAddress->get('City'),               'Kln',        'Order1.BillingAddress.City' );
    is( $BillingAddress->get('Country')->{'Code2'}, 'DE',           'Order1.BillingAddress.Country' );

    # shipping address
    my $ShippingAddress = $Order->get('ShippingAddress');
    isa_ok( $ShippingAddress, 'DE_EPAGES::Address::API::Object::Address', 'Order1.ShippingAddress' );
    is( $ShippingAddress->get('FirstName'),          'Tom',          'Order1.ShippingAddress.FirstName' );
    is( $ShippingAddress->get('LastName'),           'Tester',       'Order1.ShippingAddress.LastName' );
    is( $ShippingAddress->get('DisplayName'),        'Tom Tester',   'Order1.ShippingAddress.DisplayName' );
    is( $ShippingAddress->get('Street'),             'Hansaring 44', 'Order1.ShippingAddress.Street' );
    is( $ShippingAddress->get('Street2'),            '4. Stock',     'Order1.ShippingAddress.Street2' );
    is( $ShippingAddress->get('Zipcode'),            '50670',        'Order1.ShippingAddress.ZipCode' );
    is( $ShippingAddress->get('City'),               'Kln',        'Order1.ShippingAddress.City' );
    is( $ShippingAddress->get('Country')->{'Code2'}, 'DE',           'Order1.ShippingAddress.Country' );

    # lineitems
    my $aChildren = $Container->childrenByClass('LineItemMeinPaket');
    is(scalar @$aChildren, 1, '1 LineItemMeinPaket');
    my $Child = $aChildren->[0];
    isa_ok( $Child, 'DE_EPAGES::MeinPaket::API::Object::LineItemMeinPaket', 'Child1' );
    my $Product = $Child->get('Product');
    ok( $Product == undef, 'Child1 - No product assigned' );
    is( $Child->get('SKU'),           'de_3201212002',    'Child1 SKU: de_3201212002' );
    is( $Child->get('Name'),          'Deuter Hydro 2.0', 'Child1.Name' );
    is( $Child->get('TaxRate'),       0.19,               'Child1.TaxRate' );
    is( $Child->get('Quantity'),      1,                  'Child1.Quantity' );
    is( $Child->get('BasePrice'),     14.95,              'Child1.BasePrice' );
    is( $Child->get('LineItemPrice'), 14.95,              'Child1.LineItemPrice' );
    is( $Child->get('MeinPaketID'),   '5110439',          'Child1.MeinPaketID' );

    # check totals
    cmp_ok( $Container->get('GrandTotal'), '==', 14.95, 'Order1.GrandTotal' );

    # check second order - with product assignment
    $RemoteOrder = $Result->OrderList()->[1];
    $Order       = $aOrders->[1];
    $Container   = $Order->get('LineItemContainer');

    # general structure
    isa_ok( $RemoteOrder, 'DE_EPAGES::MeinPaket::API::DTO::Order',        'RemoteOrder' );
    isa_ok( $Order,       'DE_EPAGES::Order::API::Object::CustomerOrder', 'Order' );
    like( $Container->get('Shipping')->get( 'NameOrAlias', $LanguageID ), qr{Express}, 'Order2.Shipping "Express"' );
    cmp_ok( $Container->get('Shipping')->get('LineItemPrice'), '==', 9.00, 'Order2.Shipping.LineItemPrice' );
    is( $Container->get('Payment')->get( 'NameOrAlias', $LanguageID ), 'MeinPaket.de', 'Order2.Payment "MeinPaket.de"' );

    # billing address
    $BillingAddress = $Order->get('BillingAddress');
    isa_ok( $BillingAddress, 'DE_EPAGES::Address::API::Object::Address', 'Order2.BillingAddress' );
    is( $BillingAddress->get('Salutation'),         'Frau',             'Order2.BillingAddress.Salutation' );
    is( $BillingAddress->get('FirstName'),          'MP32',             'Order2.BillingAddress.FirstName' );
    is( $BillingAddress->get('LastName'),           'MeinPaket32',      'Order2.BillingAddress.LastName' );
    is( $BillingAddress->get('DisplayName'),        'MP32 MeinPaket32', 'Order2.BillingAddress.DisplayName' );
    is( $BillingAddress->get('Street'),             'Keuslinstr. 12',   'Order2.BillingAddress.Street' );
    is( $BillingAddress->get('Zipcode'),            '80798',            'Order2.BillingAddress.ZipCode' );
    is( $BillingAddress->get('City'),               'Mnchen',         'Order2.BillingAddress.City' );
    is( $BillingAddress->get('Country')->{'Code2'}, 'DE',               'Order2.BillingAddress.Country' );

    # shipping address
    $ShippingAddress = $Order->get('ShippingAddress');
    isa_ok( $ShippingAddress, 'DE_EPAGES::Address::API::Object::Address', 'Order2.ShippingAddress' );
    is( $ShippingAddress->get('Salutation'),         'Frau',             'Order2.ShippingAddress.Salutation' );
    is( $ShippingAddress->get('FirstName'),          'MP32',             'Order2.ShippingAddress.FirstName' );
    is( $ShippingAddress->get('LastName'),           'MeinPaket32',      'Order2.ShippingAddress.LastName' );
    is( $ShippingAddress->get('DisplayName'),        'MP32 MeinPaket32', 'Order2.ShippingAddress.DisplayName' );
    is( $ShippingAddress->get('Street'),             '1743910',          'Order2.ShippingAddress.Street' );
    is( $ShippingAddress->get('Street2'),            'Packstation 114',  'Order2.ShippingAddress.Street2' );
    is( $ShippingAddress->get('Zipcode'),            '10243',            'Order2.ShippingAddress.ZipCode' );
    is( $ShippingAddress->get('City'),               'Berlin',           'Order2.ShippingAddress.City' );
    is( $ShippingAddress->get('Country')->{'Code2'}, 'DE',               'Order2.ShippingAddress.Country' );

    # lineitems
    $aChildren = $Container->childrenByClass('LineItemMeinPaket');
    is(scalar @$aChildren, 1, '1 LineItemMeinPaket');
    $Child = $aChildren->[0];
    isa_ok( $Child, 'DE_EPAGES::MeinPaket::API::Object::LineItemMeinPaket', 'Child2' );
    $Product = $Child->get('Product');
    isa_ok( $Product, 'DE_EPAGES::Product::API::Object::Product', 'Child2.Product' );
    is( $Child->get('SKU'), 'de_3203104010', 'Child2 SKU: de_3201212002' );
    is( $Child->get( 'Name', $LanguageID ), 'Deuter Kangaroo', 'Child2.Name' );
    is( $Child->get('TaxRate'),       0.19,            'Child2.TaxRate' );
    is( $Child->get('Quantity'),      10,              'Child2.Quantity' );
    is( $Child->get('BasePrice'),     99.95,           'Child2.BasePrice' );
    is( $Child->get('LineItemPrice'), 999.50,          'Child2.LineItemPrice' );
    is( $Child->get('MeinPaketID'),   '5110441',       'Child2.MeinPaketID' );
    is( $Product->alias,              'de_3203104010', 'Child2.Product.Alias' );
    is( $Product->get('StockLevel'),  0,               'Child2.Product.StockLevel' );

    # check totals
    cmp_ok( $Container->get('GrandTotal'), '==', 1008.50, 'Order2.GrandTotal' );

    # send consignments
    BlockWarnings( sub {
        eval {
            $PortalConfig->sendNotification($aOrders, 'Consignment');
            pass('consignment no error.');
        };
        if (ExistsError()) {
            my $Error = GetError();
            fail($Error->longMessage());
        }
    });

    # check duplicate import
    $aOrders = ImportMeinPaketOrders( _loadPortalConfig(), $Result );
    is( scalar @{$aOrders}, 0, 'no duplicate import.' );

}

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            RunWithRemoteCall(sub {
                # tests
                testImportOrders();
            });

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    foreach my $MeinPaketID (1083744, 1081744) { ## no critic (RequireNumberSeparators)
        my $aPKeys = GetPKeyMeinPaketCustomerOrderMapsByMeinPaketOrderID($MeinPaketID);
        next unless (scalar @{$aPKeys});
        DeleteMeinPaketCustomerOrderMap($aPKeys->[0]);
    }
    return 1;
}

sub _loadPortalConfig {
    my $Config = LoadObjectByPath($ROOTPATH)->path( JoinObjectPath( [ 'TestShopMeinPaket', 'PortalConfigs', 'MeinPaketTestConfig' ] ) );
    isa_ok( $Config, 'DE_EPAGES::MeinPaket::API::Object::MeinPaketConfig' );
    return $Config;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( $XMLDIR.'import.xml', $ROOTPATH );

    my $Config = _loadPortalConfig();

    # initialize with our test account.
    $Config->set( {
            'PortalMerchantID'       => '3rdparty-test@epages.com',
            'PortalMerchantPassword' => 'MeinPaket2012',
        }
    );
    return 1;
}

RunScript( Sub => \&Main );

1;
