use Test::More;
use strict;
use Time::HiRes qw( gettimeofday tv_interval );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath );
use DE_EPAGES::WebInterface::API::FakeRequest;

my $SHOP_ALIAS = 'DemoShop';
my $SHOPPATH = "/Shops/$SHOP_ALIAS";

sub LoginToshop {
    my ($FakeRequest, $Shop, $URL) = @_;

    my $UrlParams = join('&', (
        'ChangeAction=SaveLoginForm',
        'Login=admin',
        'Password=admin'
    ));

    my $Response = $FakeRequest->get(
        $URL . '?' . $UrlParams,
    );
    cmp_ok( $Response->code, '==', 200, 'Login to shop response 200 OK' );

    # grep URL with sec-Token inside from content (LWP::UserAgent would do this aswell ...)
    # i.e.: <base href="https://qa-vm-lin64-1.intern.epages.de/epages/Store.admin/de_DE/">
    if( $Response->content =~ /<base\s+href=(["'])(.*?)\1/x ) { # '"
        my $SessionURL = $2;
        ok( $SessionURL =~ /$SHOP_ALIAS\.admin\/sec/g, 'Search for session URL in content');
        return $SessionURL;
    }
    else {
        die "base tag not found in\n". $Response->content;
    }
}

sub DoSmartSearch {
    my ($FakeRequest, $Shop, $SessionURL, $SearchString) = @_;

    my $UrlParams = join('&', (
        'ViewAction=JSONSmartSearchProductAlias',
        'ObjectID='.$Shop->id,
        'SearchString='.$SearchString
    ));

    my $Response = $FakeRequest->get(
        $SessionURL . '?' . $UrlParams,
        [
            'content-type' => 'application/x-www-form-urlencoded; charset=UTF-8',
            'Accept' => 'text/x-json',
        ]
    );

    # ok( $Response->is_success, '200 OK' );
    die('smart search return code OK!') unless $Response->is_success;
    # ok( $Response->content =~ /"NumResults"\s:\s/ && $Response->content =~ /"Results"\s:\s/,
    #    'Return value appears JSON' );
    die('smart search returned content not JSON! (is: '.$Response->content.')')
        unless $Response->content =~ /"NumResults"\s:\s/ && $Response->content =~ /"Results"\s:\s/;
}

sub TestProductSmartSearch {
    my ($FakeRequest, $Shop, $SessionURL) = @_;
    foreach my $SearchString ( qw(be_)) {
        my $t0= [gettimeofday];
        DoSmartSearch($FakeRequest, $Shop, $SessionURL, $SearchString);
        my $t1= [gettimeofday];
        my $elapsed = tv_interval ($t0, $t1);
        SKIP: {
      
            skip( 'Forced skip of timing tests.', 1 ) if $ENV{'SKIP_TIMING'};

            cmp_ok( $elapsed, '<', 0.2, sprintf('One product alias smart search takes less than 0.2 seconds. This run took: %0.2f seconds.', $elapsed) );
        }
    }
}

sub MassTestProductSmartSearch {
    my ($FakeRequest, $Shop, $SessionURL) = @_;

    my @SearchQueryPool = qw(be be_ be_4040 be_40401 be_404010100 be_40402 cg cg_ js js_32 md_49417110 md_494171101);
    my $t0= [gettimeofday];
#DB::enable_profile();
    for (0..200) {
        DoSmartSearch($FakeRequest, $Shop, $SessionURL, $SearchQueryPool[rand @SearchQueryPool]);
    }
#DB::disable_profile();
    my $t1= [gettimeofday];
    my $elapsed = tv_interval ($t0, $t1);
    SKIP: {
      
        skip( 'Forced skip of timing tests.', 1 ) if $ENV{'SKIP_TIMING'};
        cmp_ok( $elapsed, '<', 10, sprintf('200 random product alias smart searches take less than 10 seconds. This run took: %0.2f seconds.', $elapsed) );
    }
}

sub Main {

    if( ExistsObjectByPath($SHOPPATH) ) {
        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;

        my $Shop = LoadObjectByPath($SHOPPATH);
        ok( $Shop->id, 'load demoshop ' . $Shop->id );
        my $URL = $Shop->get('WebServerScriptAdminUrl');
        my $SessionURL = LoginToshop($FakeRequest, $Shop, $URL);

        # user clicks on product list and clicks on a product
        LoadObjectByPath($SHOPPATH)->child('Products');
        LoadObjectByPath($SHOPPATH)->child('Products')->child('be_40401');

        TestProductSmartSearch($FakeRequest, $Shop, $SessionURL);
        MassTestProductSmartSearch($FakeRequest, $Shop, $SessionURL);
    } else {
        SKIP: {
            skip 'no DemoShop installed', 8;
        };
    }
    return;
}


RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

done_testing();