/*
 * 'delimiter' don't work for DBI and 'GO' don't work with mysql command line or
 * mysql workbench. GO version is used with RunSQL.pl
 *
 */
/* delimiter $$ */
drop procedure if exists sp_ResetTaxMatrix /* $$ */
GO

create procedure sp_ResetTaxMatrix(in in_ShopID int )

    comment 'resets the tax matrix of a shop to the system tax matrix'
    LANGUAGE SQL
    MODIFIES SQL DATA
    
   /*
    *   
    *   Parameter: ShopID int
    *   What it does: 
    *   Resets the tax matrix of a shop to the system tax matrix. Some consistency 
    *   checks are made before any changes are applied.
    *
    *   Use ResetTaxMatrix.pl in the Script folder to execute this procedure.
    *   
    *   After running this script the old shop tax matrix must be deleted via deleteObject.pl
    *
    *   Remark: 
    *   Good Error Handling with SIGNAL statements requiere MySQL 5.5 or higher ;_; 
    *   Instead we use nested if-than-else 
    *   Pattern: IF (error == ture) THAN exit procedure ELSE do stuff    
    *
    */
    
begin         
       
    declare error int default 0;
  
  /*
   * This recovery tool should only run, if the conversion of Shop (SystemTaxMatrix -> CustomTaxMatrix) failed.
   * A reset on finished conversion should never executed on live system.
   * The DEBUG_MODE will disable the check if a conversion was finished.
   */
   
    declare DEBUG_MODE int default 0;  -- set 1 for debug/testing runs
    
    /*
     * If sql error occure other then 'empty set' the exit handler kicks in and does a
     * ROLLBACK (all changes are undone)
     */
     
    declare exit handler for sqlexception
    begin
        ROLLBACK;
    insert into sp_tmp_message
        select 1000, 'Error', "Unexpected sqlexception raised, catched by exit handler - ROLLBACK";
    select * from sp_tmp_message order by sortorder;
    DROP TABLE IF EXISTS sp_tmp_message;
    end;
    
    
    START TRANSACTION;
    
    /* Table sp_tmp_message is used to pass status log to caller.
     *  Perl won't recognize messages passed with SELECT 'LOG MESSAGE'. 
     *  This table is not dropped by this procedure. Caller must take care of this.
     */
    
    DROP TABLE IF EXISTS sp_tmp_message; 
    CREATE TABLE sp_tmp_message ( sortorder int(8) PRIMARY KEY, msg_type char(8), message char(128) );
    
    if not exists (select 1 from shop where shopid = in_ShopID ) then
      set error = 1;
    
      insert into sp_tmp_message
      select 10, 'Error' , concat ("ShopID ", in_ShopID, " not found. Wrong Number?");

    else       

      -- find the current shop tax matrix
      
      SELECT a.attributeid        
      FROM class c
        INNER JOIN attribute a ON a.attributeclassid = c.classid
        INNER JOIN object ao ON ao.objectid = a.attributeid
        INNER JOIN object co ON co.objectid = c.classid
        INNER JOIN object cpo ON cpo.objectid = co.parentid
      WHERE
        cpo.alias = 'Classes'
        AND cpo.parentid = 1
        AND co.alias = 'Shop'
        AND ao.alias = 'TaxMatrix'
      INTO
        @ShopTaxMatrixAttributeID;
     
      insert into sp_tmp_message
      select 20, 'Info', concat("ShopTaxMatrixAttributeID = ", IFNULL(@ShopTaxMatrixAttributeID, 'NULL'));
      
      SELECT ra.referencedobjectid
      FROM referenceattribute ra
      WHERE ra.objectid = in_ShopID
        AND ra.attributeid = @ShopTaxMatrixAttributeID
      INTO
        @ShopTaxMatrixID;
        
      insert into sp_tmp_message
      select 30, 'Info', concat("ShopTaxMatrixID = ", IFNULL(@ShopTaxMatrixAttributeID, 'NULL'));
      
      -- get the system tax matrix
      SELECT  a.attributeid
      FROM class c
        INNER JOIN attribute a ON a.attributeclassid = c.classid
        INNER JOIN object ao ON ao.objectid = a.attributeid
        INNER JOIN object co ON co.objectid = c.classid
        INNER JOIN object cpo ON cpo.objectid = co.parentid
      WHERE
        cpo.alias = 'Classes'
        AND cpo.parentid = 1
        AND co.alias = 'System'
        AND ao.alias = 'TaxMatrix'
      INTO 
        @SystemTaxMatrixAttributeID;
        
      insert into sp_tmp_message
      select 40, 'Info', concat("SystemTaxMatrixAttributeID = ",IFNULL(@SystemTaxMatrixAttributeID, 'NULL'));
      
      SELECT ra.referencedobjectid
      FROM referenceattribute ra
      WHERE ra.objectid = 1
        AND ra.attributeid = @SystemTaxMatrixAttributeID
      INTO
        @SystemTaxMatrixID;
    
      insert into sp_tmp_message      
      select 50, 'Info', concat("SystemTaxMatrixID = ", IFNULL(@SystemTaxMatrixAttributeID, 'NULL'));
           
           
      if @SystemTaxMatrixID IS NULL then
    
        insert into sp_tmp_message
        select 60, 'Error', 'System tax matrix not found';
    
        set error = 1;
      
      else
      
        if @ShopTaxMatrixID IS NOT NULL AND @SystemTaxMatrixID <> @ShopTaxMatrixID AND DEBUG_MODE = 0 then
    
          insert into sp_tmp_message
          select 70, 'Error', "Tax is already completely customized. Recovery not possible !!!";
      
          set error = 1;
          
        else   
        
          insert into sp_tmp_message      
          SELECT 70, 'Info', concat(shoptmo.alias ," = " ,shoptmo.objectid)
          FROM object shoptmo
          WHERE shoptmo.parentid = in_ShopID
            AND shoptmo.alias LIKE 'TaxMatrix%';
            
          set @OldShopTaxMatrixID = 0;
          
          SELECT shoptmo.objectid
          FROM object shoptmo
          WHERE shoptmo.parentid = in_ShopID
            AND shoptmo.alias LIKE 'TaxMatrix%'
          INTO
            @OldShopTaxMatrixID;  
          
          if @OldShopTaxMatrixID is null or @OldShopTaxMatrixID = 0 then   
      
            insert into sp_tmp_message      
            select 80, 'Error', "Old shop tax matrix not found (tax matrix has not been customized)";
            set error = 1;
            
          else
      
            insert into sp_tmp_message      
            SELECT 90, 'Info', concat(alias," (ObjectID: ", objectid, ") as ShopTaxMatrix selected")
            FROM object WHERE objectid = @OldShopTaxMatrixID;
            
            update customer c
                INNER JOIN taxarea shopta   ON shopta.taxareaid = c.taxareaid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shopta.taxmatrixid
                INNER JOIN object shoptao   ON shoptao.objectid = shopta.taxareaid
                INNER JOIN object systao    ON systao.alias = shoptao.alias
                INNER JOIN taxarea systa    ON systa.taxareaid = systao.objectid
            set c.taxareaid = systa.taxareaid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systa.taxmatrixid = @SystemTaxMatrixID;
             
             
            insert into sp_tmp_message       
            select 100, 'Info', concat(row_count()," customers updated");
            
            UPDATE lineitemcontainer lic
                INNER JOIN taxarea shopta ON shopta.taxareaid = lic.taxareaid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shopta.taxmatrixid
                INNER JOIN object shoptao  ON shoptao.objectid = shopta.taxareaid
                INNER JOIN object systao   ON systao.alias = shoptao.alias
                INNER JOIN taxarea systa ON systa.taxareaid = systao.objectid
            SET lic.taxareaid = systa.taxareaid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systa.taxmatrixid = @SystemTaxMatrixID;
        
            insert into sp_tmp_message      
            select 110, 'Info', concat(row_count()," lineitemcountainers updated");
            
            UPDATE product p
                INNER JOIN taxclass shoptc ON shoptc.taxclassid = p.taxclassid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
                INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
                INNER JOIN object systco   ON systco.alias = shoptco.alias
                INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
            SET p.taxclassid = systc.taxclassid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systc.taxmatrixid = @SystemTaxMatrixID;
          
            insert into sp_tmp_message      
            select 120, 'Info' , concat(row_count()," products updated");
          
            UPDATE lineitem li
            INNER JOIN taxclass shoptc ON shoptc.taxclassid = li.taxclassid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
                INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
                INNER JOIN object systco   ON systco.alias = shoptco.alias
                INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
            SET li.taxclassid = systc.taxclassid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systc.taxmatrixid = @SystemTaxMatrixID;
        
            insert into sp_tmp_message      
            select 130, 'Info', concat(row_count()," lineitems updated");
            
            UPDATE paymentmethod pm
                INNER JOIN taxclass shoptc ON shoptc.taxclassid = pm.taxclassid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
                INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
                INNER JOIN object systco   ON systco.alias = shoptco.alias
                INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
            SET pm.taxclassid = systc.taxclassid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systc.taxmatrixid = @SystemTaxMatrixID;
        
            insert into sp_tmp_message      
            select 140, 'Info', concat(row_count()," paymentmethods updated");
          
            UPDATE shippingmethod sm
                INNER JOIN taxclass shoptc ON shoptc.taxclassid = sm.taxclassid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
                INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
                INNER JOIN object systco   ON systco.alias = shoptco.alias
                INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
            SET sm.taxclassid = systc.taxclassid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systc.taxmatrixid = @SystemTaxMatrixID;
        
            insert into sp_tmp_message      
            select 150, 'Info', concat(row_count()," shippingmethods updated");
          
            UPDATE shippingoption so
                INNER JOIN taxclass shoptc ON shoptc.taxclassid = so.taxclassid
                INNER JOIN taxmatrix shoptm ON shoptm.taxmatrixid = shoptc.taxmatrixid
                INNER JOIN object shoptco  ON shoptco.objectid = shoptc.taxclassid
                INNER JOIN object systco   ON systco.alias = shoptco.alias
                INNER JOIN taxclass systc ON systc.taxclassid = systco.objectid
            SET so.taxclassid = systc.taxclassid
            WHERE shoptm.taxmatrixid = @OldShopTaxMatrixID
                AND systc.taxmatrixid = @SystemTaxMatrixID;
        
            insert into sp_tmp_message        
            select 160, 'Info', concat(row_count()," shippingoptions updated");
              
          UPDATE referenceattribute
          SET referencedobjectid = @SystemTaxMatrixID
          WHERE objectid = in_ShopID
            AND attributeid = @ShopTaxMatrixAttributeID;
                      
          end if; 
          
        end if;
                 
      end if;
      
    end if;
    
    if error = 0 then 
      -- don't change output 'OldShopTaxMatrixID|someID|', it is used for capturing @OldShopTaxMatrixID in the calling
      -- perl script (Tax/Scripts/ResetTaxMatrix.pl)
    insert into sp_tmp_message      
      select 170, 'Info', concat('OldShopTaxMatrixID = ', @OldShopTaxMatrixID);
      set @@autocommit = 1;
    select * from sp_tmp_message order by sortorder;
      COMMIT;
      
    else
    insert into sp_tmp_message      
      select 180, 'Error', concat ("Not all precondition met for running this procedure successfully -- ROLLBACK");
      set @@autocommit = 1;
    select * from sp_tmp_message order by sortorder;
      ROLLBACK;
      
    end if;
    
end /* $$ */
GO
/* delimiter ; */

